      SUBROUTINE CCM200(INT,LI,R,LR,KCRP,DTIM,SIG,T,EPSC,
     *                  DMG,FIE,RATB,IE)
**********************************************************************
*                                                                    *
*   Compute creep deformation and damage using Bina's model.         *
*     (KCRP>200)                                                     *
*                                                                    *
*     Input:                                                         *
*   KCRP                21x,22x,23x Bina 2a),2b),2c)                 *
*                       2x1,2x2,2x3 2x4 strain,time,damage time      *
*                                           and damage strain        *
*   DTIM                time increment                               *
*   SIG                 stress                                       *
*   T                   temperature                                  *
*   EPSC                creep strain                                 *
*   DMG                 damage                                       *
*   CTIM/CMAT/          current time                                 *
*   IE                  number of element                            *
*                                                                    *
*     Output:                                                        *
*   EPSC                creep strain                                 *
*   DMG                 damage                                       *
*   FIE                 d(strain rate)/d(sig) at CTIM                *
*   RATB                strain rate at CTIM                          *
*                                                                    *
**********************************************************************
C
      IMPLICIT INTEGER*4 (I-N), REAL*8 (A-H,O-Z)
      DIMENSION INT(LI),R(LR),A(99)
C
      COMMON /CMAT/ NMAT,LBRMP,CTIM,LBMATQ,LBIMAT,LBKIND,NBINP
C     Zjistit cislo materialu elementu
      IMAT = INT(LBIMAT+IE-1)
C     do pole A nacist materialove konstanty
      DO 1000,I=1,NBINP
        A(I)=R(LBRMP+(IMAT-1)*NBINP+I-1)
 1000 CONTINUE
C     typ vypoctu poc deformace
      KIND = INT(LBKIND+IMAT-1)
C
      KEY   = ABS(KCRP) - 200
      IF(KEY.GT.10 .AND. KEY.LT.14) KEY = KEY - 10
      IF(KEY.GT.20 .AND. KEY.LT.24) KEY = KEY - 20
      IF(KEY.GT.30 .AND. KEY.LT.34) KEY = KEY - 30
      KEY = KEY + KIND*10
C
C       FROM SI UNITS
C
      SIG   = SIG/1000000
      T     = T + 273
      EPSC  = 100*EPSC
      DENOM = 1 + EXP(-2.D0)
C
C---------------------------- CONSTANTS ------------------------------
C
C       INITIAL STRAIN
      E  = A(1) + A(2)*EXP(-A(3)/T)
      E0 = 100*SIG/E
      IF(KEY.GT.10 .AND. KEY.LT.15) KEY = KEY - 10
      IF(KEY.GT.20 .AND. KEY.LT.25) THEN
         KEY = KEY - 20
         E0  = E0*EXP( A(19)*TANH(A(21)*SIG)*EXP(A(20)/T**A(22)) )
         END IF
      IF(KEY.GT.30 .AND. KEY.LT.35) THEN
         KEY  = KEY - 30
         SIGM = A(22) + A(23)*EXP(A(24)/T)
         EM   = A(25) + A(26)*T + A(27)*T**2 + A(28)*T**3 + A(29)*T**4
         E0   = E0*EXP( A(19)*(SIG/SIGM)**EM*EXP(A(20)/T**A(21)) )
         END IF
C
C       CREEP STRENGTH
C
      AX  = MAX( ABS(1/T - 1/A(8)), 1.D-12 )
      AY  = MAX( A(9)*SIG*T,        1.D-12 )
      X   = LOG10(AX)
      Y   = LOG10( SINH(AY) )
      Z   = TANH(AY)
      TR  = A(4) + A(5)*X + A(6)*X*Y + A(7)*Y
      TR  = 10**TR
      DTR = A(9)*T*(A(7) + A(6)*X)/Z
C
C       RUPTURE STRAIN
      X    = (LOG(TR) - A(12) - A(13)*T)/A(14)
      EPSM = EXP( A(10) + A(11)*TANH(X) ) + 100*SIG/E
      DEM  = (1 - E0/EPSM)/COSH(X)**2
      DEM  = E0/EPSM/SIG + DEM*DTR*A(11)/A(14)
C---------------------------------------------------------------------
C
C       COMPUTE DAMAGES
C
      AK = EXP(A(17)+A(18)/T)
C
      IF(KEY.EQ.1) THEN
         G1   = LOG(1+EPSC/E0)/LOG(EPSM/E0)
         DMG1 = GINV(G1,A(15),A(16),AK)
         END IF
      IF(KEY.EQ.2) DMG1 = CTIM/TR
      IF(KEY.EQ.3) DMG1 = DMG
        IF(KEY.EQ.4) THEN
           EPS = DMG*(EPSM - E0)
           G   = LOG(1+EPS/E0)/LOG(EPSM/E0)
         DMG1 = GINV(G,A(15),A(16),AK)
         END IF
      DMG2 = DMG1 + DTIM/TR
C
C       UPDATE CREEP STRAIN AND DAMAGE
      G1   = DMG1**A(15)*( (1 + EXP(-2*DMG1**AK))/DENOM )**A(16)
      G2   = DMG2**A(15)*( (1 + EXP(-2*DMG2**AK))/DENOM )**A(16)
      EPS1 = E0*(EPSM/E0)**G1
      EPS2 = E0*(EPSM/E0)**G2
      EPSC = EPSC + EPS2 - EPS1
      DMG  = DMG  + DMG2 - DMG1
      IF(KEY.EQ.4) THEN
        DMG = DMG + (EPS2 - EPS1)/(EPSM - E0)
      END IF
C
C       COMPUTE DERIVATIVES
      IF(G1.LT.0.1) THEN
         G1   = 0.1
         DMG1 = 0.00248177
         END IF
      X    =  LOG(EPSM/E0)
      Y    =  EXP( -2*DMG1**AK )
      Z    =  2*A(16)*AK*DMG1**AK*Y/(1 + Y)
      D1G  =  G1*(A(15) - Z)/DMG1
      D2G  =  G1*AK*(1 - 2*DMG1**AK + Y)/(1 + Y)/D1G/DMG1
      D2G  =  DMG1*D1G/G1 - 1 - D2G*Z
      RATB =  EPS1*D1G*X/TR
      FIE  =  G1 + 1/X
      FIE  = -DTR*(1+D2G+DMG1*D1G*X) - (FIE-1)/MAX(SIG,1.D0) + DEM*FIE
      FIE  =  FIE*RATB
C
C       TO SI UNITS
      SIG  = 1000000*SIG
      T    = T - 273
      EPSC = EPSC/100
      FIE  = FIE/100000000
      RATB = RATB/100
      END
      SUBROUTINE TCD(INT,LI,R,LR,IDDAT,KCRP,KIND)
*********************************************************************************
*                                                                               *
*    Nacitani creepovych dat                                                    *
*    (Transform Creep Data)                                                     *
*                                                                               *
*   Funkce cte hlavni DAT soubor a urcuje z nej prirazeni cisel materialu       *
*    jednotlivym elementum.                                                     *
*    Prirazeni je ulozeno do souboru BEN (format shodny                         *
*                                            pro vsechny creepove modely)       *
*   Dale nacita jednotlive materialove soubory a parametry uklada               *
*    do souboru BIN (format se lisi podle typu modelu)                          *
*   Do CPMD ulozi na pozici INEXT(34) max. pocet materialovych parametru        *
*      tato hodnota slouzi pro program HPLS k alokovani pole                    *
*      zmena oproti predchozimu zpusobu vypoctu, kde pocet parametru=29         *
*           je vynucena creepovymi modely 2,3,4, ktere nemaji stanoveny max.    *
*           pocet parametru                                                     *
*                                                                               *
*       IDDAT  - hlavni DAT soubor (input)                                      *
*       KCRP   - typ creepovejho modelu (input)                                 *
*       KIND   - typ vypoctu poc. deformace - pro Binuv model (input)           *
*                                                                               *
*  COMMON /CMAT/ :                                                              *
*       NMAT     number of materials                                            *
*       LBRMP    begin of array with all material's parameters                  *
*       CTIM     creep time                                                     *
*       LBMATQ   begin of array with number of parameters for every             *
*                    material model                                             *
*       LBIMAT   begin of array with number of material for every element       *
*       LBKIND   begin of array with material model types                       *
*       NBINP    max. pocet materialovych parametru (output)                    *
*********************************************************************************
      IMPLICIT INTEGER*4 (I-N), REAL*8 (A-H,O-Z)
C
      DIMENSION INT(LI),R(LR),A(121)
C
      CHARACTER CH*8,FILES(99)*24
      CHARACTER CNAM*24,CEXT*4
C
      COMMON /CPMD/ MW,ICP(129),ICC(30),ICW(160),RCP(48),RCW(336)
      COMMON /CMAT/ NMAT,LBRMP,CTIM,LBMATQ,LBIMAT,LBKIND,NBINP
      COMMON /NAME/ CNAM,CEXT
      COMMON /NNN / LNAM
      EQUIVALENCE (ICP(9),NELEM)

C     vynulovat pocet parametru
      NBINP=0
C
C  Defaul assign material 1
C
      DO 1000,I=1,NELEM
         INT(LBIMAT+I-1) = 1
 1000 CONTINUE
C
      INT(LBKIND) = KIND
C
C  Precist NMAT nazvu souboru s materialy
C
      DO 1020,I=1,NMAT
        READ(IDDAT,*)FILES(I)
 1020 CONTINUE
C
C  Pokud je pouze jeden material, tak se preskoci dalsi nacitani, nebot
C   v souboru uz nic jineho neni
C
      IF(NMAT.EQ.1) THEN
        GO TO 5
      END IF
C
C  Assigning creep parameters to elements
C
      DO 1050,J = 2,NMAT
C       precteni radku s udaji:  material n_mat n_kind n_elem
        READ(IDDAT,*,ERR=7)CH,NUM,INT(LBKIND+J-1),N
C       kontrola cisla materialu
C       cislo materialu musi byt vetsi nez 1 a mensi rovno NMAT
        IF(NUM.LE.1.OR.NUM.GT.NMAT) THEN
         WRITE(MW,7813) NUM, NMAT
         WRITE(*,7813) NUM, NMAT
         STOP 99
        END IF
C
C       nacteni seznamu elementu; N hodnot z IDDAT ulozeno do INT
        CALL IDATA(N,IDDAT,NERR, INT(LBIMAT+NELEM),NNUM)
        IF(NERR.GT.0)THEN
          WRITE(MW,7817) J
          WRITE(*,7817) J
          IF(NNUM.LT.N) THEN
            WRITE(MW,7818) N, NNUM
            WRITE(*,7818) N, NNUM
           ENDIF
          STOP 99
        ENDIF
C
C       debug tisky - seznam elementu pro dany material
        IF(ICC(8).GT.1) THEN
         WRITE(MW,'(/32X''ELEMENTS FOR MATERIAL NUMBER'', I5)') J
         WRITE(MW,'(2X,10I8)')(INT(I),I=LBIMAT+NELEM,(LBIMAT+NELEM+N-1))
        ENDIF
C
C       kontrola zadanych cisel elementu
C       pres vsechny elementy
        DO 2050,I = 1,N
C         nacteni cisla elementu
          IE = INT(LBIMAT+NELEM+I-1)
C         pokud je cislo vetsi nez pocet elementu => chyba
          IF(IE.LT.1.OR.IE.GT.NELEM)THEN
             WRITE(MW,7815) IE,NELEM
             WRITE(*,7815) IE,NELEM
             STOP 99
          ENDIF
C         ukazatel na element
          K = LBIMAT + IE - 1
C         jestlize cislo elementu je vetsi nez 1, tak jde o duplicitni zapis elementu => chyba
          IF(INT(K).NE.1)THEN
            WRITE(MW,7816) IE
            WRITE(*,7816) IE
            STOP 99
          ENDIF
C         zapsani cisla materialu na pozici
          INT(K)= NUM
 2050   CONTINUE
C
 1050 CONTINUE
C
C   jsou precteny vsechny udaje ze souboru => uzavreni
C
    5 CLOSE(IDDAT)
C     -------------------------------------------------------
C     Output of the material parameters into the file with extension .BIN
C
      CNAM = CNAM(1:LNAM)//'.BIN'
C     otevreni souboru a zapsani poctu materialu
      OPEN(100,FILE = CNAM,FORM = 'UNFORMATTED')
        REWIND 100
        WRITE(100) NMAT
        IF(ICC(8).GT.1) THEN
         WRITE(MW,'(/32X''NUMBER OF CREEP MATERIALS:'', I5)')NMAT
        END IF
C     cteni materialovych dat podle typu creepu
      KCRP = ABS(KCRP)
C     Norton model
      IF(KCRP.EQ.2) THEN
        DO 1072,I = 1,NMAT
          WRITE(MW,'(/''Material no. '', I5)') I
C         nacteni dat ze souboru DAN
          CALL TCD2(MW,FILES(I),A,INT(LBMATQ+I-1))
C         ulozeni nejvetsiho cisla poctu nactenych dat
          IF(INT(LBMATQ+I-1).GT.NBINP) NBINP=INT(LBMATQ+I-1)
C         ulozeni do souboru BIN
          WRITE(100) INT(LBMATQ+I-1),(A(K),K=1,INT(LBMATQ+I-1))
C         testovaci tisk
          IF(ICC(8).GT.1) THEN
            WRITE(MW,'(/5X''NORTON MATERIAL NUMBER '',I3,
     *'' PROPERTIES'')') I
            WRITE(MW,*) INT(LBMATQ+I-1)
            WRITE(MW,*) (A(K),K=1,INT(LBMATQ+I-1))
          END IF
 1072   CONTINUE
      END IF
C     Norton-Baileyuv model
      IF(KCRP.EQ.3) THEN
        DO 1073,I = 1,NMAT
          WRITE(MW,'(/''Material no. '', I5)') I
C         nacteni dat ze souboru DAB
          CALL TCD3(MW,FILES(I),A,INT(LBMATQ+I-1))
C         ulozeni nejvetsiho cisla poctu nactenych dat
          IF(INT(LBMATQ+I-1).GT.NBINP) NBINP=INT(LBMATQ+I-1)
C         ulozeni do souboru BIN
          WRITE(100) INT(LBMATQ+I-1),(A(K),K=1,INT(LBMATQ+I-1))
C         testovaci tisk
          IF(ICC(8).GT.1) THEN
            WRITE(MW,'(/5X''NORTON-BAILEY MATERIAL NUMBER '',I3,
     *'' PROPERTIES'')') I
            WRITE(MW,*) INT(LBMATQ+I-1),(A(K),K=1,INT(LBMATQ+I-1))
          END IF
 1073   CONTINUE
      END IF
C     Time Hardening model
      IF(KCRP.EQ.4) THEN
        DO 1074,I = 1,NMAT
          WRITE(MW,'(/''Material no. '', I5)') I
C         nacteni dat ze souboru DAB
          CALL TCD4(MW,FILES(I),A,INT(LBMATQ+I-1))
C         ulozeni nejvetsiho cisla poctu nactenych dat
          IF(INT(LBMATQ+I-1).GT.NBINP) NBINP=INT(LBMATQ+I-1)
C         ulozeni do souboru BIN
          WRITE(100) INT(LBMATQ+I-1),(A(K),K=1,INT(LBMATQ+I-1))
C         testovaci tisk
          IF(ICC(8).GT.1) THEN
            WRITE(MW,'(/5X''TIME HARDENING MATERIAL NUMBER '',I3,
     *'' PROPERTIES'')') I
            WRITE(MW,*) INT(LBMATQ+I-1),(A(K),K=1,INT(LBMATQ+I-1))
          END IF
 1074   CONTINUE
      END IF
C     Omega model
      IF(KCRP.EQ.5) THEN
        DO 1075,I = 1,NMAT
          WRITE(MW,'(/''Material no. '', I5)') I
C         nacteni dat ze souboru DAO
          CALL TCD5(MW,FILES(I),A,INT(LBMATQ+I-1))
          IF(INT(LBMATQ+I-1).GT.NBINP) NBINP=INT(LBMATQ+I-1)
C         ulozeni do souboru BIN
          WRITE(100) (A(K),K=1,INT(LBMATQ+I-1))
C         testovaci tisk
          IF(ICC(8).GT.1) THEN
            WRITE(MW,'(/5X''OMEGA MATERIAL NUMBER '',I3,
     *'' PROPERTIES'')') I
            WRITE(MW,*) (A(K),K=1,INT(LBMATQ+I-1))
          END IF
 1075   CONTINUE
      END IF
C     Binuv model
      IF(KCRP.GT.200) THEN
        DO 1070,I = 1,NMAT
C         nacteni dat ze souboru DAT
          CALL TCD200(MW,INT(LBKIND+I-1),FILES(I),A,INT(LBMATQ+I-1))
          IF(INT(LBMATQ+I-1).GT.NBINP) NBINP=INT(LBMATQ+I-1)
C         ulozeni do souboru BIN
          WRITE(100) INT(LBMATQ+I-1),(A(K),K=1,INT(LBMATQ+I-1)),
     *               INT(LBKIND+I-1)
C         testovaci tisk
          IF(ICC(8).GT.1) THEN
            WRITE(MW,'(/5X''BINA MATERIAL NUMBER '',I3,
     *'' PROPERTIES'')') I
            WRITE(MW,*) INT(LBMATQ+I-1),(A(K),K=1,INT(LBMATQ+I-1)),
     *             INT(LBKIND+I-1)
          END IF
 1070   CONTINUE
      END IF
      CLOSE(100)
C     -------------------------------------------------------
C     ulozeni cisel materialu pro vsechny elementy do .BEL
      CNAM = CNAM(1:LNAM)//'.BEL'
      OPEN(100,FILE = CNAM,FORM = 'UNFORMATTED')
        REWIND 100
        WRITE(100)(INT(I),I=LBIMAT,(LBIMAT+NELEM-1))
      CLOSE(100)
C
      IF(ICC(8).GT.1) THEN
        WRITE(MW,'(/5X''WRITE MATERIAL ASSIGN
     *FOR ELEMENTS TO NAME.BEL'')')
        DO 1090,K=1,NELEM/10
          WRITE(MW,'(2X,20I3)')(INT(I),
     *               I=(LBIMAT+10*(K-1)),(LBIMAT+10*K-1))
 1090   CONTINUE
      END IF
      RETURN
C
    7 WRITE(MW,7814)
      WRITE(*,7814)
      STOP 99
C
 7813 FORMAT(/32X,'* ERROR IN MAIN DAT FILE *',
     *       /32x,'MATERIAL NO. ',I2,' IS NOT ALLOWED',
     *       /32x,'ONLY IN INTERVAL <2;',I2,'>')
 7814 FORMAT(/32X,'* ERROR IN READING DAT FILE*')
 7815 FORMAT(/32X,'* ERROR IN DAT FILE*',/30X,
     *      'ELEMENT', I8,' GREATER THAN NELEM =',I8)
 7816 FORMAT(/32X,'* ERROR IN DAT FILE*',/20X,
     *      'DUPLICIT ASSIGN OF MATERIAL FOR ELEMENTS ',I8)
 7817 FORMAT(/32X,'* ERROR IN READING DAT FILE*',/20X,
     *      'IN SET OF ELEMENTS FOR MATERIAL NUMBER',I2)
 7818 FORMAT(20X,'ELEMENTS REQUIRE:', I8,/20X,
     *      '         READING:',I8)
      RETURN
      END
      SUBROUTINE TCD200(MW,KIND,CNAM,A,LA)
************************************************************************
*                                                                      *
*       Cteni dat Binova modelu ze souboru DAT                         *
*       (Transform Creep Data model 200)                               *
*                                                                      *
*       MW   - log soubor (input)                                      *
*       KIND - typ vypoctu poc. deformace (input)                      *
*       CNAM - file name with parameters of Bina's creep model         *
*              (input)                                                 *
*       A    - vector with parameters of Bina's creep model            *
*              (output)                                                *
*       LA   - length of the vector A (output)                         *
************************************************************************
      IMPLICIT INTEGER*4 (I-N), REAL*8 (A-H,O-Z)
      DIMENSION A(121)
      CHARACTER CNAM*24,CID*2
      COMMON /CMAT/ NMAT,LBRMP,CTIM,LBMATQ,LBIMAT,LBKIND,NBINP
      OPEN(99,FILE = CNAM,STATUS = 'OLD')
              REWIND 99
C
    1 READ(99,'(A2)',END=78) CID
      IF(CID(1:2).NE.'PO'.AND. CID(1:2).NE.'IN') GO TO 1
      READ(99,*) A(1)
      READ(99,*) A(2)
      READ(99,*) A(3)
      READ(99,'()')
      WRITE(MW,'(/,''Initial strain E(1)-E(3)'')')
      WRITE(MW,6000)(A(K),K=1,3)
      LA = 18
C
      IF(KIND.EQ.2) THEN
         READ(99,'(A2)',END=78) CID
         IF(CID(1:2).NE.'2B') GO TO 78
         READ(99,*) A(19)
         READ(99,*) A(20)
         READ(99,*) A(21)
         READ(99,*) A(22)
         READ(99,'()')
         WRITE(MW,'(''(2b) A,Q,B,n'')')
         WRITE(MW,6000)(A(K),K=19,22)
         LA = 22
         END IF
C
      IF(KIND.EQ.3) THEN
         READ(99,'(A2)',END=78) CID
         IF(CID(1:2).NE.'2C') GO TO 78
         READ(99,*) A(19)
         READ(99,*) A(20)
         READ(99,*) A(21)
         READ(99,*) A(22)
         READ(99,*) A(23)
         READ(99,*) A(24)
         READ(99,*) A(25)
         READ(99,*) A(26)
         READ(99,*) A(27)
         READ(99,*) A(28)
         READ(99,*) A(29)
         READ(99,'()')
         WRITE(MW,'(''(2c) A,Q,n,B(1)-B(3),N(1)-N(5)'')')
         WRITE(MW,6000)(A(K),K=19,29)
         LA = 29
         END IF
C
      READ(99,'(A2)',END=78) CID
      IF(CID(1:2).NE.'PE'.AND. CID(1:2).NE.'CR') GO TO 78
      READ(99,*) A(4)
      READ(99,*) A(5)
      READ(99,*) A(6)
      READ(99,*) A(7)
      READ(99,*) A(8)
      READ(99,*) A(9)
      READ(99,'()')
      WRITE(MW,'(''Creep strength A(1)-A(6)'')')
      WRITE(MW,6000)(A(K),K=4,9)
C
      READ(99,'(A2)',END=78) CID
      IF(CID(1:2).NE.'ME'.AND. CID(1:2).NE.'RU') GO TO 78
      READ(99,*) A(10)
      READ(99,*) A(11)
      READ(99,*) A(12)
      READ(99,*) A(13)
      READ(99,*) A(14)
      READ(99,'()')
      WRITE(MW,'(''Rupture strain M(1)-M(5)'')')
      WRITE(MW,6000)(A(K),K=10,14)
C
      READ(99,'(A2)',END=78) CID
      IF(CID(1:2).NE.'FU'.AND. CID(1:2).NE.'HA') GO TO 78
      READ(99,*) A(15)
      READ(99,*) A(16)
      READ(99,*) A(17)
      READ(99,*) A(18)
      CLOSE (99)
      WRITE(MW,'(''Hardening function N,M,K1,K2'')')
      WRITE(MW,6000)(A(K),K=15,18)
C
      RETURN
 6000 FORMAT(5E15.7)
   78 WRITE(MW,7818) CNAM
      WRITE(*,7818) CNAM
      STOP 99
 7818 FORMAT(/10X,'* ERROR IN FORMAT DAT FILE ',A,'*')
      END

