      SUBROUTINE FRULE2(INT,LI,R,LR,SIG,F)
**********************************************************************
*                                                                    *
*                          Flow rule 2D.                             *
*                                                                    *
*     Input:                                                         *
*   SIG(1:4)     sig(ij) ... stress                                  *
*   SIG(5:8)     h(ij)   ... backstress                              *
*   SIG( 9)      epsp    ... cummulative plastic strain              *
*   SIG(10)      T       ... temperature                             *
*   SIG(11)      epsc    ... cummulative creep strain                *
*   SIG(12)      0       ... not defined                             *
*   SIG(13)      0       ... not defined                             *
*   SIG(14)      0       ... not defined                             *
*   SIG(15)      0       ... not defined                             *
*   SIG(16)      0       ... isotropic yield strength (DDH)          *
*                                                                    *
*   F(1:4)       stress increment                                    *
*   F(5:8)       strain increment                                    *
*                                                                    *
*     Output:                                                        *
*   SIG(1:4)     stress                                              *
*   SIG(5:8)     backstress                                          *
*   SIG( 9)      cummulative plastic strain                          *
*   SIG(10)      temperature                                         *
*                                                                    *
**********************************************************************
C
      IMPLICIT INTEGER*4 (I-N), REAL*8 (A-H,O-Z)
C
      DIMENSION INT(LI),R(LR),SIG(16),F(8),GRAD(4),RHO(4),VMP(30)
C
      COMMON /CPMD/ ICP(160),ICW(160),RCP(48),RCW(336)
C
      EQUIVALENCE (RCW(266),E1),(RCW(267),E2),(RCW(268),E3),
     *            (ICP(130),KSS),(RCP(38),DT),(ICP(116),KMOD)
C
      CALL PLCON2(INT,LI,R,LR,SIG,FNEW,SIGY,GRAD,G,FT,RHO,A,Q,VMP)
C
C       COMPUTE D*RHO
      F(5)  = E1*RHO(1) + E2*(RHO(2) + RHO(3))
      F(6)  = E1*RHO(2) + E2*(RHO(3) + RHO(1))
      F(7)  = E1*RHO(3) + E2*(RHO(1) + RHO(2))
      F(8)  = E3*RHO(4)
C
C       PLANE STRESS
      IF(KSS.EQ.-1) THEN
                    F(5) = F(5) - E2*RHO(3)
                    F(6) = F(6) - E2*RHO(3)
                    F(7) = 0
                    END IF
C
C       COMPUTE EPSP
      CALL SCALP(GRAD,F(1),4,FDE)
      CALL SCALP(GRAD,F(5),4,FDR)
      DEPS = (FDE + FT*DT)/(A + FDR)
      SIG(9)  = SIG(9)  + DEPS
      SIG(10) = SIG(10) + DT
C
C       PRAGER'S MODEL
      IF(Q.NE.0) THEN
                 DH = Q*DEPS/G**2
                 SIG(5) = SIG(5) + GRAD(1)*DH
                 SIG(6) = SIG(6) + GRAD(2)*DH
                 SIG(7) = SIG(7) + GRAD(3)*DH
                 SIG(8) = SIG(8) + GRAD(4)*DH*0.5
                 END IF
C
C       DIRECTIONAL DISTORTIONAL HARDENING MODEL
      IF (ABS(KMOD).LT.4) GOTO 100
      RHO(4) = 5.D-1*RHO(4)
C       Backstress evolution for DDH
      CALL ELIM(RHO(1),SIG(5),4,SQRT(1.5D0)*VMP(13))
      CALL MULT(RHO(1),4,DEPS*VMP(12))
      CALL ELIM(SIG(5),RHO(1),4,-1.D0)
C       Evolution of isotropic parameter k
      SIG(16) = SIG(16)+DEPS/G*VMP(10)*SQRT(3.75D-1)*
     *          (1.D0-VMP(11)*SIG(16))
C       END OF DIRECTIONAL DISTORTIONAL HARDENING
C
C       COMPUTE STRESS INCREMENT
  100 SIG(1) = SIG(1) + F(1) - DEPS*F(5)
      SIG(2) = SIG(2) + F(2) - DEPS*F(6)
      SIG(3) = SIG(3) + F(3) - DEPS*F(7)
      SIG(4) = SIG(4) + F(4) - DEPS*F(8)
C
C       RADIAL RETURN CORRECTION
      CALL PLCON2(INT,LI,R,LR,SIG,FNEW,SIGY,GRAD,G,FT,RHO,A,Q,VMP)
  200 CORR = FNEW/G**2
      SIG(1) = SIG(1) - CORR*GRAD(1)
      SIG(2) = SIG(2) - CORR*GRAD(2)
      SIG(3) = SIG(3) - CORR*GRAD(3)
      SIG(4) = SIG(4) - CORR*GRAD(4)*0.5
      IF(ABS(KMOD).EQ.4) THEN
         CALL PLCON2(INT,LI,R,LR,SIG,FNEW,SIGY,GRAD,G,FT,RHO,A,Q,VMP)
         IF(ABS(FNEW).GT.1.D-4*SIGY) GOTO 200
      ENDIF
C
      END
      SUBROUTINE FRULE3(INT,LI,R,LR,SIG,F)
**********************************************************************
*                                                                    *
*                          Flow rule 3D.                             *
*                                                                    *
*     Input:                                                         *
*   SIG(1: 6)    sig(ij) ... stress                                  *
*   SIG(7:12)    h(ij)   ... backstress                              *
*   SIG(13)      epsp    ... cummulative plastic strain              *
*   SIG(14)      T       ... temperature                             *
*   SIG(15)      epsc    ... cummulative creep strain                *
*   SIG(16)      0       ... isotropic yield strength (DDH)          *
*                                                                    *
*   F(1: 6)      stress increment                                    *
*   F(7:12)      strain increment                                    *
*                                                                    *
*     Output:                                                        *
*   SIG(1: 6)    stress                                              *
*   SIG(7:12)    backstress                                          *
*   SIG(13)      cummulative plastic strain                          *
*   SIG(14)      temperature                                         *
*                                                                    *
**********************************************************************
C
      IMPLICIT INTEGER*4 (I-N), REAL*8 (A-H,O-Z)
C
      DIMENSION INT(LI),R(LR),SIG(16),F(12),GRAD(6),RHO(6),VMP(30)
C
      COMMON /CPMD/ ICP(160),ICW(160),RCP(48),RCW(336)
C
      EQUIVALENCE (RCP(38),DT),(RCW(266),E1),(RCW(267),E2),
     *            (RCW(268),E3),(ICP(116),KMOD)
C
      CALL PLCON3(INT,LI,R,LR,SIG,FNEW,SIGY,GRAD,G,FT,RHO,A,Q,VMP)
C
C       COMPUTE D*RHO
      F(7)  = E1*RHO(1) + E2*(RHO(2) + RHO(3))
      F(8)  = E1*RHO(2) + E2*(RHO(3) + RHO(1))
      F(9)  = E1*RHO(3) + E2*(RHO(1) + RHO(2))
      F(10) = E3*RHO(4)
      F(11) = E3*RHO(5)
      F(12) = E3*RHO(6)
C
C       COMPUTE EPSP
      CALL SCALP(GRAD,F(1),6,FDE)
      CALL SCALP(GRAD,F(7),6,FDR)
      DEPS = (FDE + FT*DT)/(A + FDR)
      SIG(13) = SIG(13) + DEPS
      SIG(14) = SIG(14) + DT
C
C       PRAGER'S MODEL
      IF(Q.NE.0) THEN
                 DH = Q*DEPS/G**2
                 SIG(7)  = SIG(7)  + GRAD(1)*DH
                 SIG(8)  = SIG(8)  + GRAD(2)*DH
                 SIG(9)  = SIG(9)  + GRAD(3)*DH
                 SIG(10) = SIG(10) + GRAD(4)*DH*0.5
                 SIG(11) = SIG(11) + GRAD(5)*DH*0.5
                 SIG(12) = SIG(12) + GRAD(6)*DH*0.5
                 END IF
C
C       DIRECTIONAL DISTORTIONAL HARDENING MODEL
      IF (ABS(KMOD).LT.4) GOTO 100
      RHO(4) = 5.D-1*RHO(4)
      RHO(5) = 5.D-1*RHO(5)
      RHO(6) = 5.D-1*RHO(6)
C       Backstress evolution for DDH
      CALL ELIM(RHO(1),SIG(7),6,SQRT(1.5D0)*VMP(13))
      CALL MULT(RHO(1),6,DEPS*VMP(12))
      CALL ELIM(SIG(7),RHO(1),6,-1.D0)
C       Evolution of isotropic parameter k
      SIG(16) = SIG(16)+DEPS/G*VMP(10)*SQRT(3.75D-1)*
     *          (1.D0-VMP(11)*SIG(16))
C       END OF DIRECTIONAL DISTORTIONAL HARDENING
C
C       COMPUTE STRESS INCREMENT
  100 SIG(1) = SIG(1) + F(1) - DEPS*F(7)
      SIG(2) = SIG(2) + F(2) - DEPS*F(8)
      SIG(3) = SIG(3) + F(3) - DEPS*F(9)
      SIG(4) = SIG(4) + F(4) - DEPS*F(10)
      SIG(5) = SIG(5) + F(5) - DEPS*F(11)
      SIG(6) = SIG(6) + F(6) - DEPS*F(12)
C
C       RADIAL RETURN CORRECTION
      CALL PLCON3(INT,LI,R,LR,SIG,FNEW,SIGY,GRAD,G,FT,RHO,A,Q,VMP)
  200 CORR = FNEW/G**2
      CALL ELIM(SIG(1),GRAD(1),3,CORR)
      CALL ELIM(SIG(4),GRAD(4),3,CORR*0.5)
      IF (ABS(KMOD).EQ.4) THEN
        CALL PLCON3(INT,LI,R,LR,SIG,FNEW,SIGY,GRAD,G,FT,RHO,A,Q,VMP)
        IF(ABS(FNEW).GT.1.D-4*SIGY) GOTO 200
      ENDIF
C
      END
      SUBROUTINE PLCON2(INT,LI,R,LR,SIG,F,Y,GRAD,G,FT,S,A,Q,VMP)
*********************************************************************
*                                                                   *
*                     Plasticity condition 2D.                      *
*                                                                   *
*                F  =  HMH - Y(epsp,T,sigm,J3) [Pa]                 *
*                                                                   *
*     Input:                                                        *
*   SIG(1:4)     sig(ij) ... stress                                 *
*   SIG(5:8)     h(ij)   ... backstress                             *
*   SIG( 9)      epsp    ... cummulative plastic strain             *
*   SIG(10)      T       ... temperature                            *
*   SIG(11)      epsc    ... cummulative creep strain               *
*   SIG(12)      0       ... not defined                            *
*   SIG(13)      0       ... not defined                            *
*   SIG(14)      0       ... not defined                            *
*   SIG(15)      0       ... not defined                            *
*   SIG(16)      0       ... isotropic yield strength (DDH)         *
*                                                                   *
*   KMOD /CPMD/  < 0 perfectly plastic                              *
*                > 0 hardening                                      *
*                = 1 von Mises'  criterion                          *
*                = 2 generalized criterion     associated           *
*                = 3 generalized criterion, nonassociated           *
*                                                                   *
*     Output:                                                       *
*   F          HMH - Y                   ... plasticity condition   *
*   Y          H - Q                     ... yield stress           *
*   GRAD       grad(ij)                  ... outward normal         *
*   G          g = SQRT(grad,grad)       ... length of normal       *
*   FT         dF/dT                     ... temperature modulus    *
*   S          rho(ij)                   ... flow vector            *
*   A          dH/depsp                  ... hardening modulus      *
*   Q          dQ/depsp                  ... kinematic modulus      *
*                                                                   *
*     variables /CPMD/:                                             *
*   VX(1)    x                                        (KMOD = 1)    *
*   VX(2)    y                                        (KMOD = 1)    *
*   VX(3)    z                                        (KMOD = 1)    *
*   VX(1)    sigm = sig(kk)                           (KMOD>= 2)    *
*   VX(2)    cos(3*theta) = 3/2*SQRT(3)*J3/J2**(3/2)  (KMOD>= 2)    *
*   VX(3)    z                                        (KMOD>= 2)    *
*   VX(4)    t                                                      *
*   VX(5)    T                                                      *
*   VX(6)    DT                                                     *
*   VX(7)    epsp = INT(gamma*SQRT(depsp,depsp))                    *
*   VX(8)    Wp   = INT(sig(ij)*depsp(ij))                          *
*   VX(9)    HMH  = SQRT(3*J2) = SQRT(3/2*(sig-h),(sig-h))          *
*   VX(10)   epsc = INT(SQRT(2/3*depsc,depsc))                      *
*                                                                   *
*     variables VMP(30):                                            *
*   VMP(1)    E                                                     *
*   VMP(2)    alpha                                                 *
*   VMP(3)    nu                                                    *
*   VMP(4)    rho                                                   *
*   VMP(5)    sig_Y                                                 *
*   VMP(6)    sig_Q                                                 *
*   VMP(7)    epsc DOT                                              *
*   VMP(8)    PHI                                                   *
*                                                                   *
*********************************************************************
C
      IMPLICIT INTEGER*4 (I-N), REAL*8 (A-H,O-Z)
C
      DIMENSION INT(LI),R(LR),   SIG(16),GRAD(4),S(4),  VMP(30),VVX(10)
      DIMENSION TNR(4),DFDA(4)
C
      COMMON /CPMD/ MW,ICP(93),INEXT(36),ICC(30),
     *LIN,LRE,LIDEN,IE,IER,ITE,IME,NNE,NG,NIVEC,LIINE,LINDF,LINTC,LIADD,
     1IINE(20),INDF(20),INTC(20),IADD(10),NRVEC,LXE,LTHE,LADD,  IDUM(2),
     2JIEA,JINA,    LBMS,LBVS,LBSS,LBLS,LBNS,   KMA,KVA,KSA,KLA,KNA,KET,
     3JFS,JFL,JFN,NSA,NLA,NNA,NIG,NEG,IG,JS,JL,JN,IS,IL,IN,IEU, IWA(41),
     4RCC(30),ALFSP,PIVAL,ERAL,STEP,STEPB,TIME,RPM,           RNEXT(11),
     5VMPIG(6),VVQIG(6),VSQEG(6),VLQEG(6),VNQN(6),               TE(20),
     6VX(10),CTRD,DCX,DCY,DCZ,THIG,SF(20),XE(60),DER(60),B(60),E1,E2,E3,
     7RAUX(68)
C
      EQUIVALENCE (INEXT(3),KAVT),(INEXT(22),KMOD)
C
      CALL RMOV(VX(1),VVX(1),10)
      A  = 0
      Q  = 0
      FT = 0
      FM = 0
      FC = 0
C
C       SUBTRACT BACKSTRESS
      CALL RMOV(SIG(1),GRAD(1),4)
      CALL ELIM(GRAD(1),SIG(5),4,1.D0)
C
C       COMPUTE STRESS INVARIANTS
      SIGM = (GRAD(1) + GRAD(2) + GRAD(3))/3.
      GRAD(1) =  GRAD(1) - SIGM
      GRAD(2) =  GRAD(2) - SIGM
      GRAD(3) =  GRAD(3) - SIGM
      CALL SCALP(GRAD(1),GRAD(1),3,SIGE)
      HMH  =  SQRT(1.5*SIGE + 3.0*GRAD(4)*GRAD(4))
      COS3 =  0
      IF(ABS(KMOD).GE.2) THEN
         SIGT =  GRAD(1)*GRAD(2)*GRAD(3) - GRAD(4)*GRAD(4)*GRAD(3)
         IF(HMH.GT.0) COS3  =  13.5*SIGT/HMH**3
         VX(1) = SIGM
         VX(2) = COS3
         END IF
C
C       PLASTICITY CONDITION
      VX(5) = SIG(10)
      VX(7) = SIG(9)
      CALL VALF(INT,LI,R,LR,   LBMS,IINE,LIINE,   VMP,KMA)
      IF (ABS(KMOD).LT.4) THEN
C
      YB = VMP(5)
      QB = VMP(6)
      PHI = VMP(8)
      Y  = YB - QB
      F  = HMH - Y
C
C       COMPUTE DERIVATIVES
      IF(KMOD.GT.0) THEN
         VX(7) = VX(7) + 1.D-6
         CALL VALF(INT,LI,R,LR,   LBMS,IINE,LIINE,   VMP,KMA)
         VX(7) = VX(7) - 1.D-6
         A = (VMP(5) - YB)/1.D-6
         Q = (VMP(6) - QB)/1.D-6
         END IF
      IF(KAVT.GT.0) THEN
         VX(5) = VX(5) + 1.
         CALL VALF(INT,LI,R,LR,  LBMS,IINE,LIINE,  VMP,KMA)
         VX(5) = VX(5) - 1.
         FT = Y - VMP(5) + VMP(6)
         END IF
      IF(ABS(KMOD).GE.2) THEN
         VX(1) = VX(1) + 1.D5
         CALL VALF(INT,LI,R,LR,  LBMS,IINE,LIINE,  VMP,KMA)
         VX(1) = VX(1) - 1.D5
         FM = (Y - VMP(5) + VMP(6))/1.D5
         VX(2) = VX(2) + 1.D5
         CALL VALF(INT,LI,R,LR,  LBMS,IINE,LIINE,  VMP,KMA)
         VX(2) = VX(2) - 1.D5
         FC = (Y - VMP(5) + VMP(6))/1.D5
         END IF
C
C       OUTWARD NORMAL
      CALL RMOV(GRAD(1),S(1),4)
      IF(HMH.GT.0) CALL MULT(GRAD(1),4,1.5D0/HMH)
      G = 1.2247448D0
      IF(ABS(KMOD).GE.2) THEN
         GRAD(1) = GRAD(1) + FM/3.
         GRAD(2) = GRAD(2) + FM/3.
         GRAD(3) = GRAD(3) + FM/3.
         IF(HMH.GT.0) THEN
            C1 = 60.75*FC*SIGT/HMH**5
            C2 = 13.50*FC/HMH**3
            CALL ELIM(GRAD(1),S(1),4,C1)
            GRAD(1) = GRAD(1) + C2*(S(1)*S(1) + S(4)*S(4))
            GRAD(2) = GRAD(2) + C2*(S(2)*S(2) + S(4)*S(4))
            GRAD(3) = GRAD(3) + C2* S(3)*S(3)
            GRAD(4) = GRAD(4) + C2*(S(1) + S(2))*S(4)
            GRAD(1) = GRAD(1) - 3*FC/HMH
            GRAD(2) = GRAD(2) - 3*FC/HMH
            GRAD(3) = GRAD(3) - 3*FC/HMH
            END IF
         CALL SCALP(GRAD(1),GRAD(1),3,SIGE)
         G = SQRT(SIGE + 2*GRAD(4)*GRAD(4))
         END IF
C
      ELSE
C          DIRECTIONAL DISTORTIONAL HARDENING
C          S=(s-alpha)
         CALL RMOV(GRAD(1),S(1),4)
C          TNR=nr
         CALL RMOV(S(1),TNR(1),4)
         G=SQRT(S(1)*S(1)+S(2)*S(2)+S(3)*S(3)+2*S(4)*S(4))
C
         IF (G.GT.0) CALL MULT(TNR(1),4,1.D0/G)
C          D=c(nr:alpha)
         D=VMP(14)*(TNR(1)*SIG(5)+TNR(2)*SIG(6)+TNR(3)*SIG(7)+
     *     2*TNR(4)*SIG(8))
C          DFDA=df/dalpha
         CALL RMOV(SIG(5),DFDA(1),4)
         CALL MULT(DFDA(1),4,VMP(14))
         CALL ELIM(DFDA(1),S(1),4,VMP(14))
         CALL ELIM(DFDA(1),TNR(1),4,2.D0-D)
         CALL MULT(DFDA(1),4,SQRT(3.75D-1/(1.D0-D)))
C          F=f
         F=G*SQRT(1.5D0*(1.D0-D))-SIG(16)
C          GRAD=df/dsig
         CALL RMOV(TNR(1),GRAD(1),4)
         CALL MULT(GRAD(1),4,2.D0-D)
         CALL ELIM(GRAD(1),SIG(5),4,VMP(14))
         CALL MULT(GRAD(1),4,SQRT(3.75D-1/(1.D0-D)))
C          G=norm(df/dsig)
         G=SQRT(GRAD(1)*GRAD(1)+GRAD(2)*GRAD(2)+GRAD(3)*GRAD(3)+
     *     2.D0*GRAD(4)*GRAD(4))
C
C          Approximation of sigY by using k0
         Y=VMP(9)
C          Hardening Modulus A = K_p *sqrt(3/2)/abs(df/dsig)
         CALL RMOV(GRAD(1),TNR(1),4)
         CALL ELIM(TNR(1),SIG(5),4,VMP(13)*G)
         CALL MULT(TNR(1),4,VMP(12))
         A=DFDA(1)*TNR(1)+DFDA(2)*TNR(2)+DFDA(3)*TNR(3)+
     *     2*DFDA(4)*TNR(4)
         A=VMP(10)/2.D0*(1.D0-VMP(11)*SIG(16))-A
         A=SQRT(1.5D0)*A/G
         END IF
C
      GRAD(4) = 2*GRAD(4)
C
C       ASSOCIATED FLOW RULE
      IF(ABS(KMOD).NE.3) THEN
         GAMMA = 1.2247448D0
         IF(ABS(KMOD).EQ.2) GAMMA = SQRT(1+2*((FM/3.+0.5)/(FM/3.-1))**2)
         CALL RMOV(GRAD(1),S(1),4)
         CALL MULT(S(1),4,GAMMA/G)
         END IF
C
C       NONASSOCIATED FLOW RULE
      IF(ABS(KMOD).EQ.3) THEN
         GAMMA = SQRT( (1+2*((PHI+0.5)/(PHI-1))**2)/(1.5+3*PHI*PHI) )
         S(1)  = (1.5*S(1)/HMH + PHI)*GAMMA
         S(2)  = (1.5*S(2)/HMH + PHI)*GAMMA
         S(3)  = (1.5*S(3)/HMH + PHI)*GAMMA
         S(4)  =    3*S(4)*GAMMA/HMH
         END IF
C
      CALL RMOV(VVX(1),VX(1),10)
C
      END
      SUBROUTINE PLCON3(INT,LI,R,LR,SIG,F,Y,GRAD,G,FT,S,A,Q,VMP)
*********************************************************************
*                                                                   *
*                     Plasticity condition 3D.                      *
*                                                                   *
*                F  =  HMH - Y(epsp,T,sigm,J3) [Pa]                 *
*                                                                   *
*     Input:                                                        *
*   SIG(1: 6)    sig(ij) ... stress                                 *
*   SIG(7:12)    h(ij)   ... backstress                             *
*   SIG(13)      epsp    ... cummulative plastic strain             *
*   SIG(14)      T       ... temperature                            *
*   SIG(15)      epsc    ... cummulative creep strain               *
*   SIG(16)      0       ... isotropic yield strength (DDH)         *
*                                                                   *
*   KMOD /CPMD/  < 0 perfectly plastic                              *
*                > 0 hardening                                      *
*                = 1 von Mises'  criterion                          *
*                = 2 generalized criterion     associated           *
*                = 3 generalized criterion, nonassociated           *
*                                                                   *
*     Output:                                                       *
*   F          HMH - Y                   ... plasticity condition   *
*   Y          H - Q                     ... yield stress           *
*   GRAD       grad(ij)                  ... outward normal         *
*   G          g = SQRT(grad,grad)       ... length of normal       *
*   FT         dF/dT                     ... temperature modulus    *
*   S          rho(ij)                   ... flow vector            *
*   A          dH/depsp                  ... hardening modulus      *
*   Q          dQ/depsp                  ... kinematic modulus      *
*                                                                   *
*     variables /CPMD/:                                             *
*   VX(1)    x                                        (KMOD = 1)    *
*   VX(2)    y                                        (KMOD = 1)    *
*   VX(3)    z                                        (KMOD = 1)    *
*   VX(1)    sigm = sig(kk)                           (KMOD>= 2)    *
*   VX(2)    cos(3*theta) = 3/2*SQRT(3)*J3/J2**(3/2)  (KMOD>= 2)    *
*   VX(3)    z                                        (KMOD>= 2)    *
*   VX(4)    t                                                      *
*   VX(5)    T                                                      *
*   VX(6)    DT                                                     *
*   VX(7)    epsp = INT(gamma*SQRT(depsp,depsp))                    *
*   VX(8)    Wp   = INT(sig(ij)*depsp(ij))                          *
*   VX(9)    HMH  = SQRT(3*J2) = SQRT(3/2*(sig-h),(sig-h))          *
*   VX(10)   epsc = INT(SQRT(2/3*depsc,depsc))                      *
*                                                                   *
*     variables VMP(30):                                            *
*   VMP(1)    E                                                     *
*   VMP(2)    alpha                                                 *
*   VMP(3)    nu                                                    *
*   VMP(4)    rho                                                   *
*   VMP(5)    sig_Y                                                 *
*   VMP(6)    sig_Q                                                 *
*   VMP(7)    epsc DOT                                              *
*   VMP(8)    PHI                                                   *
*                                                                   *
*********************************************************************
C
      IMPLICIT INTEGER*4 (I-N), REAL*8 (A-H,O-Z)
C
      DIMENSION INT(LI),R(LR),   SIG(16),GRAD(6),S(6),  VMP(30),VVX(10)
      DIMENSION TNR(6),DFDA(6)
C
      COMMON /CPMD/ MW,ICP(93),INEXT(36),ICC(30),
     *LIN,LRE,LIDEN,IE,IER,ITE,IME,NNE,NG,NIVEC,LIINE,LINDF,LINTC,LIADD,
     1IINE(20),INDF(20),INTC(20),IADD(10),NRVEC,LXE,LTHE,LADD,  IDUM(2),
     2JIEA,JINA,    LBMS,LBVS,LBSS,LBLS,LBNS,   KMA,KVA,KSA,KLA,KNA,KET,
     3JFS,JFL,JFN,NSA,NLA,NNA,NIG,NEG,IG,JS,JL,JN,IS,IL,IN,IEU, IWA(41),
     4RCC(30),ALFSP,PIVAL,ERAL,STEP,STEPB,TIME,RPM,           RNEXT(11),
     5VMPIG(6),VVQIG(6),VSQEG(6),VLQEG(6),VNQN(6),               TE(20),
     6VX(10),CTRD,DCX,DCY,DCZ,THIG,SF(20),XE(60),DER(60),B(60),E1,E2,E3,
     7RAUX(68)
C
      EQUIVALENCE (INEXT(3),KAVT),(INEXT(22),KMOD)
C
      CALL RMOV(VX(1),VVX(1),10)
      A  = 0
      Q  = 0
      FT = 0
      FM = 0
      FC = 0
C
C       SUBTRACT BACKSTRESS
      CALL RMOV(SIG(1),GRAD(1),6)
      CALL ELIM(GRAD(1),SIG(7),6,1.D0)
C
C       COMPUTE STRESS INVARIANTS
      SIGM = (GRAD(1) + GRAD(2) + GRAD(3))/3.
      GRAD(1) =  GRAD(1) - SIGM
      GRAD(2) =  GRAD(2) - SIGM
      GRAD(3) =  GRAD(3) - SIGM
      CALL SCALP(GRAD(1),GRAD(1),3,SIGE)
      CALL SCALP(GRAD(4),GRAD(4),3,TAUE)
      HMH  =  SQRT(1.5*SIGE + 3.0*TAUE)
      COS3 =  0
      IF(ABS(KMOD).GE.2) THEN
         SIGT =  GRAD(1)*GRAD(2)*GRAD(3) + 2*GRAD(4)*GRAD(5)*GRAD(6) -
     -           GRAD(6)*GRAD(2)*GRAD(6) -   GRAD(4)*GRAD(4)*GRAD(3) -
     -           GRAD(5)*GRAD(5)*GRAD(1)
         IF(HMH.GT.0) COS3  =  13.5*SIGT/HMH**3
         VX(1) = SIGM
         VX(2) = COS3
         END IF
C
C       PLASTICITY CONDITION
      VX(5) = SIG(14)
      VX(7) = SIG(13)
      CALL VALF(INT,LI,R,LR,   LBMS,IINE,LIINE,   VMP,KMA)
      IF (ABS(KMOD).LT.4) THEN
C
      YB  = VMP(5)
      QB  = VMP(6)
      PHI = VMP(8)
      Y   = YB - QB
      F   = HMH - Y
C
C       COMPUTE DERIVATIVES
      IF(KMOD.GT.0) THEN
         VX(7) = VX(7) + 1.D-6
         CALL VALF(INT,LI,R,LR,   LBMS,IINE,LIINE,   VMP,KMA)
         VX(7) = VX(7) - 1.D-6
         A = (VMP(5) - YB)/1.D-6
         Q = (VMP(6) - QB)/1.D-6
         END IF
      IF(KAVT.GT.0) THEN
         VX(5) = VX(5) + 1.
         CALL VALF(INT,LI,R,LR,  LBMS,IINE,LIINE,  VMP,KMA)
         VX(5) = VX(5) - 1.
         FT = Y - VMP(5) + VMP(6)
         END IF
      IF(ABS(KMOD).GE.2) THEN
         VX(1) = VX(1) + 1.D5
         CALL VALF(INT,LI,R,LR,  LBMS,IINE,LIINE,  VMP,KMA)
         VX(1) = VX(1) - 1.D5
         FM = (Y - VMP(5) + VMP(6))/1.D5
         VX(2) = VX(2) + 1.D5
         CALL VALF(INT,LI,R,LR,  LBMS,IINE,LIINE,  VMP,KMA)
         VX(2) = VX(2) - 1.D5
         FC = (Y - VMP(5) + VMP(6))/1.D5
         END IF
C
C       OUTWARD NORMAL
      CALL RMOV(GRAD(1),S(1),6)
      IF(HMH.GT.0) CALL MULT(GRAD(1),6,1.5D0/HMH)
      G = 1.2247448D0
      IF(ABS(KMOD).GE.2) THEN
         GRAD(1) = GRAD(1) + FM/3.
         GRAD(2) = GRAD(2) + FM/3.
         GRAD(3) = GRAD(3) + FM/3.
         IF(HMH.GT.0) THEN
            C1 = 60.75*FC*SIGT/HMH**5
            C2 = 13.50*FC/HMH**3
            CALL ELIM(GRAD(1),S(1),6,C1)
            GRAD(1) = GRAD(1) + C2*(S(1)*S(1)+S(4)*S(4)+S(6)*S(6))
            GRAD(2) = GRAD(2) + C2*(S(4)*S(4)+S(2)*S(2)+S(5)*S(5))
            GRAD(3) = GRAD(3) + C2*(S(6)*S(6)+S(5)*S(5)+S(3)*S(3))
            GRAD(4) = GRAD(4) + C2*(S(1)*S(4)+S(4)*S(2)+S(6)*S(5))
            GRAD(5) = GRAD(5) + C2*(S(4)*S(6)+S(2)*S(5)+S(5)*S(3))
            GRAD(6) = GRAD(6) + C2*(S(1)*S(6)+S(4)*S(5)+S(6)*S(3))
            GRAD(1) = GRAD(1) - 3*FC/HMH
            GRAD(2) = GRAD(2) - 3*FC/HMH
            GRAD(3) = GRAD(3) - 3*FC/HMH
            END IF
         CALL SCALP(GRAD(1),GRAD(1),3,SIGE)
         CALL SCALP(GRAD(4),GRAD(4),3,TAUE)
         G = SQRT(SIGE + 2*TAUE)
         END IF
C
      ELSE
C          DIRECTIONAL DISTORTIONAL HARDENING
C          S=(s-alpha)
         CALL RMOV(GRAD(1),S(1),6)
C          TNR=nr
         CALL RMOV(S(1),TNR(1),6)
         G=SQRT(S(1)*S(1)+S(2)*S(2)+S(3)*S(3)+
     *     2*(S(4)*S(4)+S(5)*S(5)+S(6)*S(6)))
C
         IF (G.GT.0) CALL MULT(TNR(1),6,1.D0/G)
C          D=c(nr:alpha)
         D=VMP(14)*(TNR(1)*SIG(7)+TNR(2)*SIG(8)+TNR(3)*SIG(9)+
     *     2*(TNR(4)*SIG(10)+TNR(5)*SIG(11)+TNR(6)*SIG(12)))
C          DFDA=df/dalpha
         CALL RMOV(SIG(7),DFDA(1),6)
         CALL MULT(DFDA(1),6,VMP(14))
         CALL ELIM(DFDA(1),S(1),6,VMP(14))
         CALL ELIM(DFDA(1),TNR(1),6,2.D0-D)
         CALL MULT(DFDA(1),6,SQRT(3.75D-1/(1.D0-D)))
C          F=f
         F=G*SQRT(1.5D0*(1.D0-D))-SIG(16)
C          GRAD=df/dsig
         CALL RMOV(TNR(1),GRAD(1),6)
         CALL MULT(GRAD(1),6,2.D0-D)
         CALL ELIM(GRAD(1),SIG(7),6,VMP(14))
         CALL MULT(GRAD(1),6,SQRT(3.75D-1/(1.D0-D)))
C          G=norm(df/dsig)
         G=SQRT(GRAD(1)*GRAD(1)+GRAD(2)*GRAD(2)+GRAD(3)*GRAD(3)+
     *     2.D0*(GRAD(4)*GRAD(4)+GRAD(5)*GRAD(5)+GRAD(6)*GRAD(6)))
C          Approximation of sigY by using k0
         Y=VMP(9)
C          Hardening Modulus A = K_p *sqrt(3/2)/abs(df/dsig)
         CALL RMOV(GRAD(1),TNR(1),6)
         CALL ELIM(TNR(1),SIG(7),6,VMP(13)*G)
         CALL MULT(TNR(1),6,VMP(12))
         A=DFDA(1)*TNR(1)+DFDA(2)*TNR(2)+DFDA(3)*TNR(3)+
     *     2*(DFDA(4)*TNR(4)+DFDA(5)*TNR(5)+DFDA(6)*TNR(6))
         A=VMP(10)/2.D0*(1.D0-VMP(11)*SIG(16))-A
         A=SQRT(1.5D0)*A/G
         END IF
C
      GRAD(4) = 2*GRAD(4)
      GRAD(5) = 2*GRAD(5)
      GRAD(6) = 2*GRAD(6)
C
C       ASSOCIATED FLOW RULE
      IF(ABS(KMOD).NE.3) THEN
         GAMMA = 1.2247448D0
         IF(ABS(KMOD).EQ.2) GAMMA = SQRT(1+2*((FM/3.+0.5)/(FM/3.-1))**2)
         CALL RMOV(GRAD(1),S(1),6)
         CALL MULT(S(1),6,GAMMA/G)
         END IF
C
C       NONASSOCIATED FLOW RULE
      IF(ABS(KMOD).EQ.3) THEN
         GAMMA = SQRT( (1+2*((PHI+0.5)/(PHI-1))**2)/(1.5+3*PHI*PHI) )
         S(1)  = (1.5*S(1)/HMH + PHI)*GAMMA
         S(2)  = (1.5*S(2)/HMH + PHI)*GAMMA
         S(3)  = (1.5*S(3)/HMH + PHI)*GAMMA
         S(4)  =    3*S(4)*GAMMA/HMH
         S(5)  =    3*S(5)*GAMMA/HMH
         S(6)  =    3*S(6)*GAMMA/HMH
         END IF
C
      CALL RMOV(VVX(1),VX(1),10)
C
      END

