      SUBROUTINE CCM6(INT,LI,R,LR,DTIM,SIG,T,EPSC,DMG,
     *                        FIE,RATB,IE,THETA,EPSCC,EPSH,SIGPREV)
**********************************************************************
*                                                                    *
*   Compute creep deformation and damage using                       *
*       IPM Kloc Model (KCRP=6)                                      *
*                                                                    *
*   model potrebuje z predchoziho kroku tahe hodnoty THETA a EPSL    *
*                                                                    *
*                                                                    *
*     Input:                                                         *
*   DTIM                time increment                               *
*   SIG                 stress           [Pa]                        *
*   T                   temperature      [C]                         *
*   EPSC                creep strain                                 *
*   DMG                 damage                                       *
*   CTIM/CMAT/          current time                                 *
*   IE                  number of element                            *
*   QQ                                                               *
*   EPSL                                                             *
*                                                                    *
*     Output:                                                        *
*   EPSC [-]            creep strain                                 *
*   DMG [-]             damage                                       *
*   FIE                 ë(strain rate)/ë(sig) at CTIM                *
*   RATB                strain rate at CTIM                          *
*                                                                    *
*     Input / Output                                                 *
*   THETA                                                            *
*   EPSCC               velikost eps_c deformace                     *
*   EPSH                velikost eps_h deformace                     *
*   SIGPREV             predchozi velikost SIG                       *
*                                                                    *
**********************************************************************
C
C
      IMPLICIT INTEGER*4 (I-N), REAL*8 (A-H,O-Z)
      DIMENSION INT(LI),R(LR),PS(3),A(99)
C
      COMMON /CMAT/ NMAT,LBRMP,CTIM,LBMATQ,LBIMAT,LBKIND,NBINP
C
C     promenne subroutiny
C      C_ST                       -- materialova konstanta sigma_t [MPa]
C      C_SR                       -- materialova konstanta sigma_r [MPa]
C      C_RI                       -- materialova konstanta r_i     [-]
C      C_B                        -- materialova konstanta b       [1/s]
C      C_P                        -- materialova konstanta p       [1/MPa]
C      C_QH                       -- materialova konstanta Q_h     [kJ/mol]
C      C_QL                       -- materialova konstanta Q_l     [kJ/mol]
C      C_G                        -- materialova konstanta g       [1/s]
C      C_E                        -- materialova konstanta E       [MPa]
C      C_K                        -- materialova konstanta k       [-]
C      C_C                        -- materialova konstanta c       [-]
C      C_R                        -- univerzalni plynova konstanta [kJ/K/mol]
C      T_K                        -- teplota v Kelvinech           [K]
C      DEPSS                      -- \dot{\varepsilon}_s           [1/s]
C      DTHETA                     -- prirustek hodnoty THETA
C      DEPSL                      -- rychlost deformace pro low stress
C      DEPSH                      -- rychlost deformace pro high stress
C
C      RATE                       -- rychlost creepove deformace pouzita pro numerickou derivaci
C      THETAFIE                   -- theta pro vypocet FIE
C
C     -----------  nacteni materialovych konstant  --------------------
C
C     Zjistit cislo materialu elementu
      IMAT = INT(LBIMAT+IE-1)
C     do pole A nacist materialove konstanty
      DO 1000,I=1,NBINP
        A(I)=R(LBRMP+(IMAT-1)*NBINP+I-1)
 1000 CONTINUE
      C_ST=A(1); C_SR=A(2); C_RI=A(3); C_B=A(4); C_P=A(5);
      C_QH=A(6); C_QL=A(7); C_G=A(8);  C_E=A(9); C_K=A(10); C_C=A(11);
C     teplota z C na K
      T_K = T + 273.15
C     univerzalni plynova konstanta [kJ/K/mol]
      C_R = 0.0083144598
C
C--------------------  vypocet rychlosti deformace ---------------------
C
C     prepocet napeti na MPa
      SIG = SIG/1000000
      SIGPREV = SIGPREV/1000000
      IF(CTIM.EQ.0) SIGPREV = SIG
      SIGPREV = SIG
C     vypocet dot sigma - pro rce. (12)
      DSIGMA = (SIG - SIGPREV) / DTIM
C     napeti zvetsene o 0,1MPa pro vypocet derivace rychlosti podle napeti
      SIGFIE = SIG+0.1
C
C   vypocet high deformace DEPSH
C   vypocet dot{eps}_s - rce. (9)
      DEPSS =  C_B * EXP(-C_QH/C_R/T_K) *
     *        SINH(C_P*(2*SIG + SQRT((SIG-C_ST)*(SIG-C_ST)+C_SR*C_SR) -
     *                          SQRT((SIG+C_ST)*(SIG+C_ST)+C_SR*C_SR)))
C    prirustek theta pro dany casovy krok
      DTHETA = ABS(DEPSS) * DTIM / C_C
C    inkrementace THETA o vypocteny prirustek
      THETA = THETA + DTHETA
C    vypocet high deformace DEPSH - rce. (6)
      DEPSH = DEPSS * (1+C_RI) / (1+C_RI-C_RI*EXP(-THETA))
C    pro dalsi krok uchovat THETA

C   vypocet low deformace DEPSL
C    rce. (12) - eps_l = eps_sig + eps_cc
C    vypocet DEPSCC - rce. (11)
      DEPSCC = C_G * EXP(-C_QL/C_R/T_K) * C_K *
     *         (C_K*SIG/(C_E*(1-C_K))-EPSCC)*
     *         (C_K*SIG/(C_E*(1-C_K))-EPSCC)*
     *         (C_K*SIG/(C_E*(1-C_K))-EPSCC)

C    vypocet DEPSCCFIE - rce. (11)
      DEPSCCFIE = C_G * EXP(-C_QL/C_R/T_K) * C_K *
     *         (C_K*SIGFIE/(C_E*(1-C_K))-EPSCC)*
     *         (C_K*SIGFIE/(C_E*(1-C_K))-EPSCC)*
     *         (C_K*SIGFIE/(C_E*(1-C_K))-EPSCC)

      DEPSSIGMA = (SIG - SIGPREV)/ DTIM/ C_E

C    hodnota rychlosti - rce (13)
      RATB =  DEPSCC + DEPSH
C      + DSIGMA / C_E

C--------------------  vypocet deformace  --------------------------
C
C     vypocet eps_c deformace - rce. (11)
      EPSCC = EPSCC + DEPSCC * DTIM
C     vypocet celkove deformace
C            low                high
      EPSL = EPSCC + SIG/C_E
      EPSH = EPSH + DEPSH * DTIM
      EPSC = EPSL + EPSH

C--------------------  vypocet derivace rychlosti podle napeti  ----
C  RATE se spocita pro hodnotu napeti zvetsenou o 0,1 MPa
C      SIGFIE = SIG+0.1
C   vypocet dot{eps}_s - rce. (9)
      DEPSS =  C_B * EXP(-C_QH/C_R/T_K) *
     *        SINH(C_P*(2*SIGFIE +
     *           SQRT((SIGFIE-C_ST)*(SIGFIE-C_ST)+C_SR*C_SR) -
     *           SQRT((SIGFIE+C_ST)*(SIGFIE+C_ST)+C_SR*C_SR)))
C    inkrementace THETA
      THETAFIE = THETA - DTHETA + ABS(DEPSS) * DTIM / C_C
C    vypocet high deformace DEPSH - rce. (6)
      DEPSHFIE = DEPSS * (1+C_RI) / (1+C_RI-C_RI*EXP(-THETAFIE))
C
C    hodnota rychlosti  RATE
C      RATE =  DEPSCCFIE + DEPSHFIE
C
C   FIE       low                    high
      FIELOW = (DEPSCCFIE - DEPSCC)*10
      FIEHIGH = (DEPSHFIE - DEPSH)*10
      FIE = FIELOW + FIEHIGH
C
C--------------------  ukonceni a prevod jednotek ------------------
      DMG = RATB
C
C     TO SI UNITS
      SIG  = 1000000*SIG
      SIGPREV = SIG
      FIE = FIE/1000000
C
      END
      SUBROUTINE TCD(INT,LI,R,LR,IDDAT,KCRP,KIND)
*********************************************************************************
*                                                                               *
*    Nacitani creepovych dat                                                    *
*    (Transform Creep Data)                                                     *
*                                                                               *
*   Funkce cte hlavni DAT soubor a urcuje z nej prirazeni cisel materialu       *
*    jednotlivym elementum.                                                     *
*    Prirazeni je ulozeno do souboru BEN (format shodny                         *
*                                            pro vsechny creepove modely)       *
*   Dale nacita jednotlive materialove soubory a parametry uklada               *
*    do souboru BIN (format se lisi podle typu modelu)                          *
*   Do CPMD ulozi na pozici INEXT(34) max. pocet materialovych parametru        *
*      tato hodnota slouzi pro program HPLS k alokovani pole                    *
*      zmena oproti predchozimu zpusobu vypoctu, kde pocet parametru=29         *
*           je vynucena creepovymi modely 2,3,4, ktere nemaji stanoveny max.    *
*           pocet parametru                                                     *
*                                                                               *
*       IDDAT  - hlavni DAT soubor (input)                                      *
*       KCRP   - typ creepovejho modelu (input)                                 *
*       KIND   - typ vypoctu poc. deformace - pro Binuv model (input)           *
*                                                                               *
*  COMMON /CMAT/ :                                                              *
*       NMAT     number of materials                                            *
*       LBRMP    begin of array with all material's parameters                  *
*       CTIM     creep time                                                     *
*       LBMATQ   begin of array with number of parameters for every             *
*                    material model                                             *
*       LBIMAT   begin of array with number of material for every element       *
*       LBKIND   begin of array with material model types                       *
*       NBINP    max. pocet materialovych parametru (output)                    *
*********************************************************************************
      IMPLICIT INTEGER*4 (I-N), REAL*8 (A-H,O-Z)
C
      DIMENSION INT(LI),R(LR),A(121)
C
      CHARACTER CH*8,FILES(99)*24
      CHARACTER CNAM*24,CEXT*4
C
      COMMON /CPMD/ MW,ICP(129),ICC(30),ICW(160),RCP(48),RCW(336)
      COMMON /CMAT/ NMAT,LBRMP,CTIM,LBMATQ,LBIMAT,LBKIND,NBINP
      COMMON /NAME/ CNAM,CEXT
      COMMON /NNN / LNAM
      EQUIVALENCE (ICP(9),NELEM)

C     vynulovat pocet parametru
      NBINP=0
C
C  Defaul assign material 1
C
      DO 1000,I=1,NELEM
         INT(LBIMAT+I-1) = 1
 1000 CONTINUE
C
      INT(LBKIND) = KIND
C
C  Precist NMAT nazvu souboru s materialy
C
      DO 1020,I=1,NMAT
        READ(IDDAT,*)FILES(I)
 1020 CONTINUE
C
C  Pokud je pouze jeden material, tak se preskoci dalsi nacitani, nebot
C   v souboru uz nic jineho neni
C
      IF(NMAT.EQ.1) THEN
        GO TO 5
      END IF
C
C  Assigning creep parameters to elements
C
      DO 1050,J = 2,NMAT
C       precteni radku s udaji:  material n_mat n_kind n_elem
        READ(IDDAT,*,ERR=7)CH,NUM,INT(LBKIND+J-1),N
C       kontrola cisla materialu
C       cislo materialu musi byt vetsi nez 1 a mensi rovno NMAT
        IF(NUM.LE.1.OR.NUM.GT.NMAT) THEN
         WRITE(MW,7813) NUM, NMAT
         WRITE(*,7813) NUM, NMAT
         STOP 99
        END IF
C
C       nacteni seznamu elementu; N hodnot z IDDAT ulozeno do INT
        CALL IDATA(N,IDDAT,NERR, INT(LBIMAT+NELEM),NNUM)
        IF(NERR.GT.0)THEN
          WRITE(MW,7817) J
          WRITE(*,7817) J
          IF(NNUM.LT.N) THEN
            WRITE(MW,7818) N, NNUM
            WRITE(*,7818) N, NNUM
           ENDIF
          STOP 99
        ENDIF
C
C       debug tisky - seznam elementu pro dany material
        IF(ICC(8).GT.1) THEN
         WRITE(MW,'(/32X''ELEMENTS FOR MATERIAL NUMBER'', I5)') J
         WRITE(MW,'(2X,10I8)')(INT(I),I=LBIMAT+NELEM,(LBIMAT+NELEM+N-1))
        ENDIF
C
C       kontrola zadanych cisel elementu
C       pres vsechny elementy
        DO 2050,I = 1,N
C         nacteni cisla elementu
          IE = INT(LBIMAT+NELEM+I-1)
C         pokud je cislo vetsi nez pocet elementu => chyba
          IF(IE.LT.1.OR.IE.GT.NELEM)THEN
             WRITE(MW,7815) IE,NELEM
             WRITE(*,7815) IE,NELEM
             STOP 99
          ENDIF
C         ukazatel na element
          K = LBIMAT + IE - 1
C         jestlize cislo elementu je vetsi nez 1, tak jde o duplicitni zapis elementu => chyba
          IF(INT(K).NE.1)THEN
            WRITE(MW,7816) IE
            WRITE(*,7816) IE
            STOP 99
          ENDIF
C         zapsani cisla materialu na pozici
          INT(K)= NUM
 2050   CONTINUE
C
 1050 CONTINUE
C
C   jsou precteny vsechny udaje ze souboru => uzavreni
C
    5 CLOSE(IDDAT)
C     -------------------------------------------------------
C     Output of the material parameters into the file with extension .BIN
C
      CNAM = CNAM(1:LNAM)//'.BIN'
C     otevreni souboru a zapsani poctu materialu
      OPEN(100,FILE = CNAM,FORM = 'UNFORMATTED')
        REWIND 100
        WRITE(100) NMAT
        IF(ICC(8).GT.1) THEN
         WRITE(MW,'(/32X''NUMBER OF CREEP MATERIALS:'', I5)')NMAT
        END IF
C     cteni materialovych dat podle typu creepu
      KCRP = ABS(KCRP)
C     Norton model
      IF(KCRP.EQ.2) THEN
        DO 1072,I = 1,NMAT
          WRITE(MW,'(/''Material no. '', I5)') I
C         nacteni dat ze souboru DAN
          CALL TCD2(MW,FILES(I),A,INT(LBMATQ+I-1))
C         ulozeni nejvetsiho cisla poctu nactenych dat
          IF(INT(LBMATQ+I-1).GT.NBINP) NBINP=INT(LBMATQ+I-1)
C         ulozeni do souboru BIN
          WRITE(100) INT(LBMATQ+I-1),(A(K),K=1,INT(LBMATQ+I-1))
C         testovaci tisk
          IF(ICC(8).GT.1) THEN
            WRITE(MW,'(/5X''NORTON MATERIAL NUMBER '',I3,
     *'' PROPERTIES'')') I
            WRITE(MW,*) INT(LBMATQ+I-1)
            WRITE(MW,*) (A(K),K=1,INT(LBMATQ+I-1))
          END IF
 1072   CONTINUE
      END IF
C     Norton-Baileyuv model
      IF(KCRP.EQ.3) THEN
        DO 1073,I = 1,NMAT
          WRITE(MW,'(/''Material no. '', I5)') I
C         nacteni dat ze souboru DAB
          CALL TCD3(MW,FILES(I),A,INT(LBMATQ+I-1))
C         ulozeni nejvetsiho cisla poctu nactenych dat
          IF(INT(LBMATQ+I-1).GT.NBINP) NBINP=INT(LBMATQ+I-1)
C         ulozeni do souboru BIN
          WRITE(100) INT(LBMATQ+I-1),(A(K),K=1,INT(LBMATQ+I-1))
C         testovaci tisk
          IF(ICC(8).GT.1) THEN
            WRITE(MW,'(/5X''NORTON-BAILEY MATERIAL NUMBER '',I3,
     *'' PROPERTIES'')') I
            WRITE(MW,*) INT(LBMATQ+I-1),(A(K),K=1,INT(LBMATQ+I-1))
          END IF
 1073   CONTINUE
      END IF
C     Time Hardening model
      IF(KCRP.EQ.4) THEN
        DO 1074,I = 1,NMAT
          WRITE(MW,'(/''Material no. '', I5)') I
C         nacteni dat ze souboru DAB
          CALL TCD4(MW,FILES(I),A,INT(LBMATQ+I-1))
C         ulozeni nejvetsiho cisla poctu nactenych dat
          IF(INT(LBMATQ+I-1).GT.NBINP) NBINP=INT(LBMATQ+I-1)
C         ulozeni do souboru BIN
          WRITE(100) INT(LBMATQ+I-1),(A(K),K=1,INT(LBMATQ+I-1))
C         testovaci tisk
          IF(ICC(8).GT.1) THEN
            WRITE(MW,'(/5X''TIME HARDENING MATERIAL NUMBER '',I3,
     *'' PROPERTIES'')') I
            WRITE(MW,*) INT(LBMATQ+I-1),(A(K),K=1,INT(LBMATQ+I-1))
          END IF
 1074   CONTINUE
      END IF
C     Omega model
      IF(KCRP.EQ.5) THEN
        DO 1075,I = 1,NMAT
          WRITE(MW,'(/''Material no. '', I5)') I
C         nacteni dat ze souboru DAO
          CALL TCD5(MW,FILES(I),A,INT(LBMATQ+I-1))
          IF(INT(LBMATQ+I-1).GT.NBINP) NBINP=INT(LBMATQ+I-1)
C         ulozeni do souboru BIN
          WRITE(100) (A(K),K=1,INT(LBMATQ+I-1))
C         testovaci tisk
          IF(ICC(8).GT.1) THEN
            WRITE(MW,'(/5X''OMEGA MATERIAL NUMBER '',I3,
     *'' PROPERTIES'')') I
            WRITE(MW,*) (A(K),K=1,INT(LBMATQ+I-1))
          END IF
 1075   CONTINUE
      END IF
C     Binuv model
      IF(KCRP.GT.200) THEN
        DO 1070,I = 1,NMAT
C         nacteni dat ze souboru DAT
          CALL TCD200(MW,INT(LBKIND+I-1),FILES(I),A,INT(LBMATQ+I-1))
          IF(INT(LBMATQ+I-1).GT.NBINP) NBINP=INT(LBMATQ+I-1)
C         ulozeni do souboru BIN
          WRITE(100) INT(LBMATQ+I-1),(A(K),K=1,INT(LBMATQ+I-1)),
     *               INT(LBKIND+I-1)
C         testovaci tisk
          IF(ICC(8).GT.1) THEN
            WRITE(MW,'(/5X''BINA MATERIAL NUMBER '',I3,
     *'' PROPERTIES'')') I
            WRITE(MW,*) INT(LBMATQ+I-1),(A(K),K=1,INT(LBMATQ+I-1)),
     *             INT(LBKIND+I-1)
          END IF
 1070   CONTINUE
      END IF
      CLOSE(100)
C     -------------------------------------------------------
C     ulozeni cisel materialu pro vsechny elementy do .BEL
      CNAM = CNAM(1:LNAM)//'.BEL'
      OPEN(100,FILE = CNAM,FORM = 'UNFORMATTED')
        REWIND 100
        WRITE(100)(INT(I),I=LBIMAT,(LBIMAT+NELEM-1))
      CLOSE(100)
C
      IF(ICC(8).GT.1) THEN
        WRITE(MW,'(/5X''WRITE MATERIAL ASSIGN
     *FOR ELEMENTS TO NAME.BEL'')')
        DO 1090,K=1,NELEM/10
          WRITE(MW,'(2X,20I3)')(INT(I),
     *               I=(LBIMAT+10*(K-1)),(LBIMAT+10*K-1))
 1090   CONTINUE
      END IF
      RETURN
C
    7 WRITE(MW,7814)
      WRITE(*,7814)
      STOP 99
C
 7813 FORMAT(/32X,'* ERROR IN MAIN DAT FILE *',
     *       /32x,'MATERIAL NO. ',I2,' IS NOT ALLOWED',
     *       /32x,'ONLY IN INTERVAL <2;',I2,'>')
 7814 FORMAT(/32X,'* ERROR IN READING DAT FILE*')
 7815 FORMAT(/32X,'* ERROR IN DAT FILE*',/30X,
     *      'ELEMENT', I8,' GREATER THAN NELEM =',I8)
 7816 FORMAT(/32X,'* ERROR IN DAT FILE*',/20X,
     *      'DUPLICIT ASSIGN OF MATERIAL FOR ELEMENTS ',I8)
 7817 FORMAT(/32X,'* ERROR IN READING DAT FILE*',/20X,
     *      'IN SET OF ELEMENTS FOR MATERIAL NUMBER',I2)
 7818 FORMAT(20X,'ELEMENTS REQUIRE:', I8,/20X,
     *      '         READING:',I8)
      RETURN
      END
      SUBROUTINE TCD6(MW,CNAM,A,LA)
************************************************************************
*                                                                      *
*       Cteni dat IPM Kloc modelu ze souboru DAK                       *
*       (Transform Creep Data model 6)                                 *
*                                                                      *
*       MW   - log soubor (input)                                      *
*       CNAM - file name with parameters of Omega creep model (input)  *
*       A    - vector with parameters of IPM creep model (output)      *
*       LA   - length of the vector A = 11 (output)                    *
*                                                                      *
*                                            14.9.2017 Masak           *
************************************************************************
      IMPLICIT INTEGER*4 (I-N), REAL*8 (A-H,O-Z)
      DIMENSION A(121)
      CHARACTER CNAM*24,CID*2
      COMMON /CMAT/ NMAT,LBRMP,CTIM,LBMATQ,LBIMAT,LBKIND,NBINP
      OPEN(99,FILE = CNAM,STATUS = 'OLD')
      REWIND 99
C
    1 READ(99,'(A2)',END=78) CID
      IF(CID(1:2).NE.'KO' .AND. CID(1:2).NE.'ko') GO TO 1
      READ(99,*) A(1)
      READ(99,*) A(2)
      READ(99,*) A(3)
      READ(99,*) A(4)
      READ(99,*) A(5)
      READ(99,*) A(6)
      READ(99,*) A(7)
      READ(99,*) A(8)
      READ(99,*) A(9)
      READ(99,*) A(10)
      READ(99,*) A(11)
      WRITE(MW,'(/,''CONSTANTS'')')
      WRITE(MW,6000)(A(K),K=1,11)
C
      LA = 11
      RETURN
 6000 FORMAT(5E15.7)
   78 WRITE(MW,7818) CNAM
      WRITE(*,7818) CNAM
      STOP 99
 7818 FORMAT(/10X,'* ERROR IN FORMAT DAK FILE ',A,'*')
      END
