      SUBROUTINE CCM3(INT,LI,R,LR,DTIM,SIG,T,EPSC,DMG,FIE,RATB,IE)
************************************************************************
*                                                                      *
*   Compute creep deformation using Norton-Bailey model. (KCRP=3)      *
*                                                                      *
*     Input:                                                           *
*   DTIM                time increment                                 *
*   SIG                 stress                                         *
*   T                   temperature                                    *
*   EPSC                creep strain                                   *
*   DMG                 damage                                         *
*   CTIM/CMAT/          current time                                   *
*   IE                  number of element                              *
*                                                                      *
*     Output:                                                          *
*   EPSC                creep strain                                   *
*   DMG = 0             damage                                         *
*   FIE                 d(strain rate)/d(sig) at CTIM                  *
*   RATB                strain rate at CTIM                            *
*                                                                      *
************************************************************************
C
      IMPLICIT INTEGER*4 (I-N), REAL*8 (A-H,O-Z)
      DIMENSION INT(LI),R(LR),A(121)
C
      COMMON /CMAT/ NMAT,LBRMP,CTIM,LBMATQ,LBIMAT,LBKIND,NBINP
C
C     -----------  nacteni materialovych konstant  --------------------
C     cislo materialu
      IMAT = INT(LBIMAT+IE-1)
C     pocet zadanych datovych radku
      KDATASET = INT(LBMATQ+IMAT-1)/4
C     materialove konstatny
      DO 1000,I=1,NBINP
        A(I)=R(LBRMP+(IMAT-1)*NBINP+I-1)
 1000 CONTINUE
C     nastaveni rozmezi teplot
      T_MIN = A(1)
      T_MAX = A((KDATASET-1)*4+1)
C     kontrola teploty na meze
      IF(T.LT.T_MIN .OR. T.GT.T_MAX) THEN
         WRITE(*,'(/24X,''* ERROR in CCM3 - Norton-Bailey model *'',
     *             /24X,''Element no. '',I7,
     *             /24X,''Temperature '',F7.2,
     *            '' out of range <'',F7.2,'';'',F7.2,''>'')')
     *             IE,T,T_MIN,T_MAX
         STOP 99
        END IF
C
C     napeti na MPa
      SIG = SIG/1000000
C
C   ----------  vyhledani odpovidajiciho intervalu  ----------------
C
      DO I=2, KDATASET
        IF(T.GE.A((I-2)*4+1) .AND. T.LE.A((I-1)*4+1)) THEN
           INDEX=I-1
          END IF
      END DO
C     v nalezenem intervalu zjistit pomer T
      INDEX = (INDEX-1)*4+1
      T_RATIO = (T-A(INDEX))/(A(INDEX+4)-A(INDEX))
C
C      nastavit konstanty vypoctu
      INDEX = INDEX+1
      C_K1 = A(INDEX)
      C_N1 = A(INDEX+1)
      C_M1 = A(INDEX+2)
      INDEX = INDEX+4
      C_K2 = A(INDEX)
      C_N2 = A(INDEX+1)
      C_M2 = A(INDEX+2)
C
C   ----------  vypocet hodnot  ----------------
C
C  Creepova deformace EPSC2
C  vypocet pro obe teploty intervalu
C  pocita se pro koncovy cas aktualniho intervalu = CTIM + DTIM
C
      EPSC21 = C_K1 * SIG**C_N1 * (CTIM+DTIM)**C_M1
      EPSC22 = C_K2 * SIG**C_N2 * (CTIM+DTIM)**C_M2
C  Creepova deformace EPSC1
C  pocita se pro pocatecni cas aktualniho intervalu = CTIM
C
      EPSC11 = C_K1 * SIG**C_N1 * CTIM**C_M1
      EPSC12 = C_K2 * SIG**C_N2 * CTIM**C_M2
C     rozdil casove diference
      DEPSC1 = EPSC21 - EPSC11
      DEPSC2 = EPSC22 - EPSC12
C     vypocet hodnoty z pomeru T_RATIO
      EPSC = EPSC + (DEPSC2 - DEPSC1) * T_RATIO + DEPSC1
C
C   Rychlost
C   pocita se pro pocatecni cas CTIM
C   pro CTIM==0 (nastava v LC 2) se nastavi 1E-8
C
      TIME_RATB = CTIM
      IF(TIME_RATB.EQ.0) THEN
          TIME_RATB = 1E-8
      END IF
      RATB1 = C_K1 * SIG**C_N1 * C_M1 * TIME_RATB**(C_M1-1)
      RATB2 = C_K2 * SIG**C_N2 * C_M2 * TIME_RATB**(C_M2-1)
C     vypocet hodnoty z pomeru T_RATIO
      RATB = (RATB2 - RATB1) * T_RATIO + RATB1
C
C      EPSC1 = CONST_K * SIG**CONST_N * TIME_RATB**CONST_M
C     Opakovany vypocet EPSC1 z duvodu moznosti nuloveho casu
C      pouzije se pri vypocti FIE
      EPSC11 = C_K1 * SIG**C_N1 * TIME_RATB**C_M1
      EPSC12 = C_K2 * SIG**C_N2 * TIME_RATB**C_M2
C     vypocet hodnoty z pomeru T_RATIO
      EPSC1 = (EPSC12 - EPSC11) * T_RATIO + EPSC11
C
C  Vypocet derivace rychlosti podle napeti
C
C   koeficient mocniny u deformace je zaporny
C   pro nizke EPSC (dojde pri napeti blizke nule) dojde k preteceni
C     pocita se vyraz typu:  E-16 ^ -19 = 10^304
C     je treba pocitat pres logaritmus a omezit exponent
C      y =  EPSC1**EX_E
C    log10 y = EX_E * log10 EPSC1
C              vyraz na prave strane musi byt mensi nez 280
C
C   koeficient mocniny u napeti je kladny
C       k preteceni muze dojit jen pro velmi vysoke napeti
C
C     pro teplotu T1
      EX_E = (C_M1-1)/C_M1
      EX_S = (C_N1-C_M1)/C_M1
      EX_LOG = 1/C_M1 * LOG10(C_K1) + LOG10(C_N1) +
     *           EX_E * LOG10(EPSC1) + EX_S * LOG10(SIG)
      IF(EX_LOG.GE.300) THEN
        EX_LOG = 300
      END IF
      IF(EX_LOG.LE.-300) THEN
        EX_LOG = -300
      END IF
      FIE1 = 10**EX_LOG
C     pro teplotu T2
      EX_E = (C_M2-1)/C_M2
      EX_S = (C_N2-C_M2)/C_M2
      EX_LOG = 1/C_M2 * LOG10(C_K2) + LOG10(C_N2) +
     *           EX_E * LOG10(EPSC1) + EX_S * LOG10(SIG)
      IF(EX_LOG.GE.300) THEN
        EX_LOG = 300
      END IF
      IF(EX_LOG.LE.-300) THEN
        EX_LOG = -300
      END IF
      FIE2 = 10**EX_LOG
C     vypocet hodnoty z pomeru T_RATIO
      FIE = (FIE2 - FIE1) * T_RATIO + FIE1
C
      DMG = 0
C
C       TO SI UNITS
      SIG  = 1000000*SIG
      FIE  = FIE/1000000
C
      END
      SUBROUTINE TCD(INT,LI,R,LR,IDDAT,KCRP,KIND)
*********************************************************************************
*                                                                               *
*    Nacitani creepovych dat                                                    *
*    (Transform Creep Data)                                                     *
*                                                                               *
*   Funkce cte hlavni DAT soubor a urcuje z nej prirazeni cisel materialu       *
*    jednotlivym elementum.                                                     *
*    Prirazeni je ulozeno do souboru BEN (format shodny                         *
*                                            pro vsechny creepove modely)       *
*   Dale nacita jednotlive materialove soubory a parametry uklada               *
*    do souboru BIN (format se lisi podle typu modelu)                          *
*   Do CPMD ulozi na pozici INEXT(34) max. pocet materialovych parametru        *
*      tato hodnota slouzi pro program HPLS k alokovani pole                    *
*      zmena oproti predchozimu zpusobu vypoctu, kde pocet parametru=29         *
*           je vynucena creepovymi modely 2,3,4, ktere nemaji stanoveny max.    *
*           pocet parametru                                                     *
*                                                                               *
*       IDDAT  - hlavni DAT soubor (input)                                      *
*       KCRP   - typ creepovejho modelu (input)                                 *
*       KIND   - typ vypoctu poc. deformace - pro Binuv model (input)           *
*                                                                               *
*  COMMON /CMAT/ :                                                              *
*       NMAT     number of materials                                            *
*       LBRMP    begin of array with all material's parameters                  *
*       CTIM     creep time                                                     *
*       LBMATQ   begin of array with number of parameters for every             *
*                    material model                                             *
*       LBIMAT   begin of array with number of material for every element       *
*       LBKIND   begin of array with material model types                       *
*       NBINP    max. pocet materialovych parametru (output)                    *
*********************************************************************************
      IMPLICIT INTEGER*4 (I-N), REAL*8 (A-H,O-Z)
C
      DIMENSION INT(LI),R(LR),A(121)
C
      CHARACTER CH*8,FILES(99)*24
      CHARACTER CNAM*24,CEXT*4
C
      COMMON /CPMD/ MW,ICP(129),ICC(30),ICW(160),RCP(48),RCW(336)
      COMMON /CMAT/ NMAT,LBRMP,CTIM,LBMATQ,LBIMAT,LBKIND,NBINP
      COMMON /NAME/ CNAM,CEXT
      COMMON /NNN / LNAM
      EQUIVALENCE (ICP(9),NELEM)

C     vynulovat pocet parametru
      NBINP=0
C
C  Defaul assign material 1
C
      DO 1000,I=1,NELEM
         INT(LBIMAT+I-1) = 1
 1000 CONTINUE
C
      INT(LBKIND) = KIND
C
C  Precist NMAT nazvu souboru s materialy
C
      DO 1020,I=1,NMAT
        READ(IDDAT,*)FILES(I)
 1020 CONTINUE
C
C  Pokud je pouze jeden material, tak se preskoci dalsi nacitani, nebot
C   v souboru uz nic jineho neni
C
      IF(NMAT.EQ.1) THEN
        GO TO 5
      END IF
C
C  Assigning creep parameters to elements
C
      DO 1050,J = 2,NMAT
C       precteni radku s udaji:  material n_mat n_kind n_elem
        READ(IDDAT,*,ERR=7)CH,NUM,INT(LBKIND+J-1),N
C       kontrola cisla materialu
C       cislo materialu musi byt vetsi nez 1 a mensi rovno NMAT
        IF(NUM.LE.1.OR.NUM.GT.NMAT) THEN
         WRITE(MW,7813) NUM, NMAT
         WRITE(*,7813) NUM, NMAT
         STOP 99
        END IF
C
C       nacteni seznamu elementu; N hodnot z IDDAT ulozeno do INT
        CALL IDATA(N,IDDAT,NERR, INT(LBIMAT+NELEM),NNUM)
        IF(NERR.GT.0)THEN
          WRITE(MW,7817) J
          WRITE(*,7817) J
          IF(NNUM.LT.N) THEN
            WRITE(MW,7818) N, NNUM
            WRITE(*,7818) N, NNUM
           ENDIF
          STOP 99
        ENDIF
C
C       debug tisky - seznam elementu pro dany material
        IF(ICC(8).GT.1) THEN
         WRITE(MW,'(/32X''ELEMENTS FOR MATERIAL NUMBER'', I5)') J
         WRITE(MW,'(2X,10I8)')(INT(I),I=LBIMAT+NELEM,(LBIMAT+NELEM+N-1))
        ENDIF
C
C       kontrola zadanych cisel elementu
C       pres vsechny elementy
        DO 2050,I = 1,N
C         nacteni cisla elementu
          IE = INT(LBIMAT+NELEM+I-1)
C         pokud je cislo vetsi nez pocet elementu => chyba
          IF(IE.LT.1.OR.IE.GT.NELEM)THEN
             WRITE(MW,7815) IE,NELEM
             WRITE(*,7815) IE,NELEM
             STOP 99
          ENDIF
C         ukazatel na element
          K = LBIMAT + IE - 1
C         jestlize cislo elementu je vetsi nez 1, tak jde o duplicitni zapis elementu => chyba
          IF(INT(K).NE.1)THEN
            WRITE(MW,7816) IE
            WRITE(*,7816) IE
            STOP 99
          ENDIF
C         zapsani cisla materialu na pozici
          INT(K)= NUM
 2050   CONTINUE
C
 1050 CONTINUE
C
C   jsou precteny vsechny udaje ze souboru => uzavreni
C
    5 CLOSE(IDDAT)
C     -------------------------------------------------------
C     Output of the material parameters into the file with extension .BIN
C
      CNAM = CNAM(1:LNAM)//'.BIN'
C     otevreni souboru a zapsani poctu materialu
      OPEN(100,FILE = CNAM,FORM = 'UNFORMATTED')
        REWIND 100
        WRITE(100) NMAT
        IF(ICC(8).GT.1) THEN
         WRITE(MW,'(/32X''NUMBER OF CREEP MATERIALS:'', I5)')NMAT
        END IF
C     cteni materialovych dat podle typu creepu
      KCRP = ABS(KCRP)
C     Norton model
      IF(KCRP.EQ.2) THEN
        DO 1072,I = 1,NMAT
          WRITE(MW,'(/''Material no. '', I5)') I
C         nacteni dat ze souboru DAN
          CALL TCD2(MW,FILES(I),A,INT(LBMATQ+I-1))
C         ulozeni nejvetsiho cisla poctu nactenych dat
          IF(INT(LBMATQ+I-1).GT.NBINP) NBINP=INT(LBMATQ+I-1)
C         ulozeni do souboru BIN
          WRITE(100) INT(LBMATQ+I-1),(A(K),K=1,INT(LBMATQ+I-1))
C         testovaci tisk
          IF(ICC(8).GT.1) THEN
            WRITE(MW,'(/5X''NORTON MATERIAL NUMBER '',I3,
     *'' PROPERTIES'')') I
            WRITE(MW,*) INT(LBMATQ+I-1)
            WRITE(MW,*) (A(K),K=1,INT(LBMATQ+I-1))
          END IF
 1072   CONTINUE
      END IF
C     Norton-Baileyuv model
      IF(KCRP.EQ.3) THEN
        DO 1073,I = 1,NMAT
          WRITE(MW,'(/''Material no. '', I5)') I
C         nacteni dat ze souboru DAB
          CALL TCD3(MW,FILES(I),A,INT(LBMATQ+I-1))
C         ulozeni nejvetsiho cisla poctu nactenych dat
          IF(INT(LBMATQ+I-1).GT.NBINP) NBINP=INT(LBMATQ+I-1)
C         ulozeni do souboru BIN
          WRITE(100) INT(LBMATQ+I-1),(A(K),K=1,INT(LBMATQ+I-1))
C         testovaci tisk
          IF(ICC(8).GT.1) THEN
            WRITE(MW,'(/5X''NORTON-BAILEY MATERIAL NUMBER '',I3,
     *'' PROPERTIES'')') I
            WRITE(MW,*) INT(LBMATQ+I-1),(A(K),K=1,INT(LBMATQ+I-1))
          END IF
 1073   CONTINUE
      END IF
C     Time Hardening model
      IF(KCRP.EQ.4) THEN
        DO 1074,I = 1,NMAT
          WRITE(MW,'(/''Material no. '', I5)') I
C         nacteni dat ze souboru DAB
          CALL TCD4(MW,FILES(I),A,INT(LBMATQ+I-1))
C         ulozeni nejvetsiho cisla poctu nactenych dat
          IF(INT(LBMATQ+I-1).GT.NBINP) NBINP=INT(LBMATQ+I-1)
C         ulozeni do souboru BIN
          WRITE(100) INT(LBMATQ+I-1),(A(K),K=1,INT(LBMATQ+I-1))
C         testovaci tisk
          IF(ICC(8).GT.1) THEN
            WRITE(MW,'(/5X''TIME HARDENING MATERIAL NUMBER '',I3,
     *'' PROPERTIES'')') I
            WRITE(MW,*) INT(LBMATQ+I-1),(A(K),K=1,INT(LBMATQ+I-1))
          END IF
 1074   CONTINUE
      END IF
C     Omega model
      IF(KCRP.EQ.5) THEN
        DO 1075,I = 1,NMAT
          WRITE(MW,'(/''Material no. '', I5)') I
C         nacteni dat ze souboru DAO
          CALL TCD5(MW,FILES(I),A,INT(LBMATQ+I-1))
          IF(INT(LBMATQ+I-1).GT.NBINP) NBINP=INT(LBMATQ+I-1)
C         ulozeni do souboru BIN
          WRITE(100) (A(K),K=1,INT(LBMATQ+I-1))
C         testovaci tisk
          IF(ICC(8).GT.1) THEN
            WRITE(MW,'(/5X''OMEGA MATERIAL NUMBER '',I3,
     *'' PROPERTIES'')') I
            WRITE(MW,*) (A(K),K=1,INT(LBMATQ+I-1))
          END IF
 1075   CONTINUE
      END IF
C     Binuv model
      IF(KCRP.GT.200) THEN
        DO 1070,I = 1,NMAT
C         nacteni dat ze souboru DAT
          CALL TCD200(MW,INT(LBKIND+I-1),FILES(I),A,INT(LBMATQ+I-1))
          IF(INT(LBMATQ+I-1).GT.NBINP) NBINP=INT(LBMATQ+I-1)
C         ulozeni do souboru BIN
          WRITE(100) INT(LBMATQ+I-1),(A(K),K=1,INT(LBMATQ+I-1)),
     *               INT(LBKIND+I-1)
C         testovaci tisk
          IF(ICC(8).GT.1) THEN
            WRITE(MW,'(/5X''BINA MATERIAL NUMBER '',I3,
     *'' PROPERTIES'')') I
            WRITE(MW,*) INT(LBMATQ+I-1),(A(K),K=1,INT(LBMATQ+I-1)),
     *             INT(LBKIND+I-1)
          END IF
 1070   CONTINUE
      END IF
      CLOSE(100)
C     -------------------------------------------------------
C     ulozeni cisel materialu pro vsechny elementy do .BEL
      CNAM = CNAM(1:LNAM)//'.BEL'
      OPEN(100,FILE = CNAM,FORM = 'UNFORMATTED')
        REWIND 100
        WRITE(100)(INT(I),I=LBIMAT,(LBIMAT+NELEM-1))
      CLOSE(100)
C
      IF(ICC(8).GT.1) THEN
        WRITE(MW,'(/5X''WRITE MATERIAL ASSIGN
     *FOR ELEMENTS TO NAME.BEL'')')
        DO 1090,K=1,NELEM/10
          WRITE(MW,'(2X,20I3)')(INT(I),
     *               I=(LBIMAT+10*(K-1)),(LBIMAT+10*K-1))
 1090   CONTINUE
      END IF
      RETURN
C
    7 WRITE(MW,7814)
      WRITE(*,7814)
      STOP 99
C
 7813 FORMAT(/32X,'* ERROR IN MAIN DAT FILE *',
     *       /32x,'MATERIAL NO. ',I2,' IS NOT ALLOWED',
     *       /32x,'ONLY IN INTERVAL <2;',I2,'>')
 7814 FORMAT(/32X,'* ERROR IN READING DAT FILE*')
 7815 FORMAT(/32X,'* ERROR IN DAT FILE*',/30X,
     *      'ELEMENT', I8,' GREATER THAN NELEM =',I8)
 7816 FORMAT(/32X,'* ERROR IN DAT FILE*',/20X,
     *      'DUPLICIT ASSIGN OF MATERIAL FOR ELEMENTS ',I8)
 7817 FORMAT(/32X,'* ERROR IN READING DAT FILE*',/20X,
     *      'IN SET OF ELEMENTS FOR MATERIAL NUMBER',I2)
 7818 FORMAT(20X,'ELEMENTS REQUIRE:', I8,/20X,
     *      '         READING:',I8)
      RETURN
      END
      SUBROUTINE TCD3(MW,CNAM,A,LA)
************************************************************************
*                                                                      *
*       Cteni dat Nortonova-Baileyova modelu ze souboru DAB            *
*      (Transform Creep Data model 3)                                  *
*                                                                      *
*       MW   - log soubor (input)                                      *
*       CNAM - file name with parameters of Norton-Bailey creep model  *
*              (input)                                                 *
*       A    - vector with parameters of Norton-Bailey creep model     *
*              (output)                                                *
*       LA   - length of the vector (output)                           *
************************************************************************
      IMPLICIT INTEGER*4 (I-N), REAL*8 (A-H,O-Z)
      DIMENSION A(121)
      CHARACTER CNAM*24,CID*2
      COMMON /CMAT/ NMAT,LBRMP,CTIM,LBMATQ,LBIMAT,LBKIND,NBINP
C     pocet dat na radku
      NLD = 4
C     priznak typu zadavanych koeficientu
C     0 = PMD
C     1 = ANSYS (prepocitat na PMD)
      ITOPMD=0
C     otevrit soubor
      OPEN(99,FILE = CNAM,STATUS = 'OLD')
      REWIND 99
C     priznak NB MODEL
    1 READ(99,'(A2)',END=78) CID
      IF(CID(1:2).NE.'NB'.AND. CID(1:2).NE.'nb') GO TO 1
C     typ koeficientu PMD/ANSYS
      READ(99,'(A2)',END=78) CID
      IF(CID(1:2).NE.'AN'.AND.CID(1:2).NE.'an'.AND.
     *   CID(1:2).NE.'PM'.AND.CID(1:2).NE.'pm') GO TO 78
      IF(CID(1:2).EQ.'AN' .OR. CID(1:2).EQ.'an') ITOPMD=1
      READ(99,'()')
C     POCET DAT
      READ(99,'(A2)',END=78) CID
      IF(CID(1:2).NE.'PO'.AND. CID(1:2).NE.'po') GO TO 78
      READ(99,*) LA
      READ(99,'()')
      WRITE(MW,'(''Data file: '',A )') CNAM
      WRITE(MW,'(''No of data lines:'',I3 )') LA
      IF(ITOPMD.GT.0) THEN
        WRITE(MW,'(''ANSYS coeficients recalculate to PMD'')')
      END IF
C     kontrola na pocet dat
      IF(LA.GT.30) THEN
        WRITE(MW,7819) LA
        WRITE(*,7819) LA
        STOP 99
      END IF
C     priznak DATA
      READ(99,'(A2)',END=78) CID
      IF(CID(1:2).NE.'DA'.AND. CID(1:2).NE.'da') GO TO 78
      WRITE(MW,'(''Datas:'')')
C     cteni dat
      DO 1000 II = 1, LA
        READ(99,*) A(NLD*(II-1)+1), A(NLD*(II-1)+2), A(NLD*(II-1)+3),
     *                A(NLD*(II-1)+4)
        IF(ITOPMD.EQ.1) THEN
          A(NLD*(II-1)+2) = A(NLD*(II-1)+2)/(A(NLD*(II-1)+4)+1)
          A(NLD*(II-1)+4) = A(NLD*(II-1)+4) + 1
        END IF
      WRITE(MW,6000)(A(K),K=NLD*(II-1)+1,NLD*(II-1)+NLD)
 1000 CONTINUE
C     pokud je zadana pouze jedna teplota, je treba vytvorit 2 zaznamy,
C     jeden s teplotou o 1C nizsi a jeden s teplotou o 1C vyssi
C     Subroutina CCM3 predpoklada, ze jsou zadany alespon 2 teploty
      IF(LA.EQ.1) THEN
C        snizit teplotu o 1C
         A(1) = A(1) - 1
C        zapsat druhy material
         A(NLD+1) = A(1) + 2
         A(NLD+2) = A(2)
         A(NLD+3) = A(3)
         A(NLD+4) = A(4)
         LA = 2
      END IF
      LA = LA*4
      RETURN
 6000 FORMAT(5E15.7)
   78 WRITE(MW,7818) CNAM
      WRITE(*,7818) CNAM
      STOP 99
 7818 FORMAT(/10X,'* ERROR IN FORMAT DAB FILE ',A,'*')
 7819 FORMAT(/32X,'* ERROR IN DAB FILE *',
     *       /32x,'NO. OF DATAS LINE ',I3,' IS GREATER THAN 30')
      END

